#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "fitsio.h"
#include "fitsdebug.h"

	 
int fd_comment(fd_file *file,  /* structure holding fits file      */
	       char *comment,  /* Comment to be added to fits file */ 
	       int *status) {  

  fd_flush_open(file);

  if (fits_write_comment(file->fptr, comment, status)) 
    return *status;
  else
    *status = FD_STAT_ALLOK;

  fd_flush_close(file);

  return *status;

}

int fd_comment_no_flush(fd_file *file, /* structure holding fits file     */
			char *comment, /* Comment to be added to fitsfile */
			int *status) { 

  if (fd_get_file_status(file) == FD_FILE_OPEN) {
    if (fits_write_comment(file->fptr, comment, status)) 
      return *status;
    else
      *status = FD_STAT_ALLOK;
  } else {
    *status = FD_STAT_FILE_NOT_OPEN;
  }

  return *status;

}

int fd_set_ckeyword(fd_file *file,  /* structure holding fits file      */
		    char *keyword,  /* keyword to be added to fits file */ 
		    char *value,    /* Value of keyword to be added */
		    char *comment,
		    int *status) {  

  char *tmp_key;

  tmp_key = mem_malloc_safe((strlen(keyword)+1)*sizeof(char));
  str_upcase(tmp_key,keyword);

  fd_flush_open(file);

  if (fits_write_key(file->fptr, TSTRING, tmp_key, value, comment, status)) {
    free(tmp_key);
    return *status;
  } else {
    *status = FD_STAT_ALLOK;
  }

  free(tmp_key);
  fd_flush_close(file);

  return *status;

}

int fd_set_ckeyword_no_flush(fd_file *file, /* structure holding fits file  */
		 	     char *keyword, /* keyword to be added to file */ 
			     char *value,   /* Value of keyword to be added */
			     char *comment,
			     int *status) { 
 
  char *tmp_key;

  tmp_key = mem_malloc_safe((strlen(keyword)+1)*sizeof(char));
  str_upcase(tmp_key,keyword);

  if (fd_get_file_status(file) == FD_FILE_OPEN) {
    if (fits_write_key(file->fptr, TSTRING, tmp_key, value, 
		       comment, status)) { 
      free(tmp_key);
      return *status;
    } else {
      *status = FD_STAT_ALLOK;
    }
  } else {
    *status = FD_STAT_FILE_NOT_OPEN;
  }
  
  free(tmp_key);

  return *status;

}

int fd_set_ikeyword(fd_file *file,  /* structure holding fits file      */
		    char *keyword,  /* keyword to be added to fits file */ 
		    int  value,     /* Value of keyword to be added */
		    char *comment,
		    int *status) {  

  char *tmp_key;

  tmp_key = mem_malloc_safe((strlen(keyword)+1)*sizeof(char));
  str_upcase(tmp_key,keyword);

  fd_flush_open(file);

  if (fits_write_key(file->fptr, TINT, tmp_key, &value, comment, status)) {
    free(tmp_key);
    return *status;
  } else {
    *status = FD_STAT_ALLOK;
  }

  free(tmp_key);
  fd_flush_close(file);

  return *status;

}

int fd_set_ikeyword_no_flush(fd_file *file, /* structure holding fits file  */
		 	     char *keyword, /* keyword to be added to file */ 
			     int value,     /* Value of keyword to be added */
			     char *comment,
			     int *status) { 
 
  char *tmp_key;

  tmp_key = mem_malloc_safe((strlen(keyword)+1)*sizeof(char));
  str_upcase(tmp_key,keyword);

  if (fd_get_file_status(file) == FD_FILE_OPEN) {
    if (fits_write_key(file->fptr, TINT, tmp_key, &value, 
		       comment, status)) { 
      free(tmp_key);
      return *status;
    } else {
      *status = FD_STAT_ALLOK;
    }
  } else {
    *status = FD_STAT_FILE_NOT_OPEN;
  }
  
  free(tmp_key);

  return *status;

}

int fd_set_fkeyword(fd_file *file,  /* structure holding fits file      */
		    char *keyword,  /* keyword to be added to fits file */ 
		    float  value,   /* Value of keyword to be added */
		    char *comment,
		    int *status) {  

  char *tmp_key;

  tmp_key = mem_malloc_safe((strlen(keyword)+1)*sizeof(char));
  str_upcase(tmp_key,keyword);

  fd_flush_open(file);

  if (fits_write_key(file->fptr, TFLOAT, tmp_key, &value, comment, status)) {
    free(tmp_key);
    return *status;
  } else {
    *status = FD_STAT_ALLOK;
  }

  free(tmp_key);
  fd_flush_close(file);

  return *status;

}

int fd_set_fkeyword_no_flush(fd_file *file, /* structure holding fits file  */
		 	     char *keyword, /* keyword to be added to file */ 
			     float value,   /* Value of keyword to be added */
			     char *comment,
			     int *status) { 
 
  char *tmp_key;

  tmp_key = mem_malloc_safe((strlen(keyword)+1)*sizeof(char));
  str_upcase(tmp_key,keyword);

  if (fd_get_file_status(file) == FD_FILE_OPEN) {
    if (fits_write_key(file->fptr, TFLOAT, tmp_key, &value, 
		       comment, status)) { 
      free(tmp_key);
      return *status;
    } else {
      *status = FD_STAT_ALLOK;
    }
  } else {
    *status = FD_STAT_FILE_NOT_OPEN;
  }
  
  free(tmp_key);

  return *status;

}

