#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fitsio.h"
#include "fitsdebug.h"

	 
int fd_open_file(fd_file *file,    /* structure holding fits file */
		 char *filename,   /* filename of file to create  */ 
		 char *tablename,  /* Name of the table           */
		 int *status,      /* Status flag                 */
		 int flush) {      /* Flush flag                  */

  /* open an existsing fitsfile.
     

  */

  char val_tmp[MAXSTRLEN], comm_tmp[MAXSTRLEN], 
    tform[6], ttype[6], tunit[6];
  fitsfile *fptr;
  /* required keywords */
  /* keywords required for the Table HDU */
  int hdunum = 0, itmp = 0;
  long ltmp = 0;


  /* Check if the file to be opened exists. If not, create an 
     error message and exit the program */
  if (!(fd_file_exist(filename))) 
    msg_exit(-1,"Error (fd_open_file): File %s does not exist\n",filename);


  /* First things first: check if all input is within the allowed
     range */

  /* check if flush flag has valid value, if not return error */
  if (!fd_set_flush(file,flush)) {
    *status = FD_STAT_FLUSH;
    return FD_STAT_FLUSH;
  }

  /* 1: create the new fits file */
  if (fits_open_file(&fptr, filename, READWRITE, status)) {
    return *status; 
  } else {
    fd_set_fptr(file,fptr);
    fd_set_filename(file,filename);
    fd_set_hdu(file,1);
    fd_set_file_status(file,FD_FILE_OPEN);
    fd_set_supersede(file,FD_SUPERSEDE_OFF);
  }

  /* 2: append creation date and a comment line to the fits file */
  sprintf(comm_tmp,"==> File opened or modified on %s",time_get_ut_string(val_tmp));
  if (fits_write_comment(fptr, comm_tmp, status)) 
    return *status;
  
  /* move to the HDU corresponding to the tablename */
  if (fits_movnam_hdu(fptr, ANY_HDU, tablename, 0, status)) {
    msg_message(0,"Could not find table %s in file %s.\n", 
		tablename, filename);
    fd_exit_file(file, status);
    exit(*status);
  }

  fd_set_hdu(file,fits_get_hdu_num(fptr, &hdunum));
  fd_set_tablename(file, tablename);

  /* get table type */
  if (fits_read_key(fptr, TSTRING, "XTENSION", val_tmp, comm_tmp, status))
    return *status;
  if (strncmp(val_tmp,"BINTABLE",8) == 0)
    fd_set_tabletype(file,BINARY_TBL);
  else if (strncmp(val_tmp,"TABLE",5) == 0)
    fd_set_tabletype(file,ASCII_TBL);
  else {
    msg_message(0,"Unknown table type [%s].\n", val_tmp);
    itmp = FD_STAT_TABLE_TYPE;
    fd_exit_file(file, &itmp);
    return FD_STAT_TABLE_TYPE;
  }

  /* get number or rows and columns */
  fits_get_num_cols(fptr, &itmp,status); fd_set_ncol(file,itmp);
  fits_get_num_rows(fptr, &ltmp,status); fd_set_nrow(file,ltmp);

  /* fill TFORM, TTYPE, and TUNIT */
  /* 1: allocate memory */
  file->tform = mem_malloc_safe(fd_get_ncol(file)*sizeof(char *));
  file->ttype = mem_malloc_safe(fd_get_ncol(file)*sizeof(char *));
  file->tunit = mem_malloc_safe(fd_get_ncol(file)*sizeof(char *));
  file->typecode = mem_malloc_safe(fd_get_ncol(file)*sizeof(typecodeS));
  /* 2: read in column by column, allocate memory and set values */
  for (itmp = 1; itmp <= fd_get_ncol(file); itmp++) {
    sprintf(tform,"TFORM%d",itmp);
    sprintf(ttype,"TTYPE%d",itmp);
    sprintf(tunit,"TUNIT%d",itmp);
  
    if (fits_read_key(fptr, TSTRING, tform, val_tmp, comm_tmp, status))
      return *status;
    else {
      file->tform[itmp-1] = mem_malloc_safe((strlen(val_tmp)+1)*sizeof(char));
      strcpy(file->tform[itmp-1],val_tmp);
    }
    
    if (fits_read_key(fptr, TSTRING, ttype, val_tmp, comm_tmp, status)) { 
      if (*status == KEY_NO_EXIST) {
	file->ttype[itmp-1] = mem_malloc_safe(sizeof(char));
	strcpy(file->ttype[itmp-1],"");
	*status = 0;
      }	else {
	return *status;
      }
    } else {
      file->ttype[itmp-1] = mem_malloc_safe((strlen(val_tmp)+1)*sizeof(char));
      strcpy(file->ttype[itmp-1],val_tmp);
    }
    
    if (fits_read_key(fptr, TSTRING, tunit, val_tmp, comm_tmp, status)) {
      if (*status == KEY_NO_EXIST) {
	file->tunit[itmp-1] = mem_malloc_safe(sizeof(char));
	strcpy(file->tunit[itmp-1],"");
	*status = 0;
      }	else {
	return *status;
      }
    } else {
      file->tunit[itmp-1] = mem_malloc_safe((strlen(val_tmp)+1)*sizeof(char));
      strcpy(file->tunit[itmp-1],val_tmp);
    }
    
    if (fd_get_tabletype(file) == BINARY_TBL) 
      fd_set_typecode_binary(&file->typecode[itmp-1],file->tform[itmp-1], 
			     status);
    else
      fd_set_typecode_ascii(&file->typecode[itmp-1],file->tform[itmp-1], 
			    status);


  }

  /* Finally: before returning check the flush flag and handle
     the request accordingly                                   */

  if (fd_flush_close(file))
    msg_message(3,"-->Error in fd_flush_close"); 

  return FD_STAT_ALLOK;

}


