#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fitsio.h"
#include "fitsdebug.h"
#include "utilities.h"


int fd_print_rows(FILE *output, fd_file *file, long row_start, long row_end) {

  long nrow = fd_get_nrow(file);
  char **final;
  int i, fsize;
  int n = 0;

  /* handle flush/opening of file */
  fd_flush_open(file);

  if (row_start == FD_ROW_START) row_start = 1;
  if (row_end   == FD_ROW_END)   row_end = nrow;


  /* check of requested rows exist in the file */
  when(nrow < row_start,
       msg_message(2,"Requested rows are out of range\nFile has %d rows\n", nrow);
       return 1;);
  
  /* check if the lower bounds of the requested rows, if invalid */
  /* adjust                                                      */
  when(row_start < 1,
       msg_message(2,"Requested rows are out of range\nChanging row_start to 1\n");
       row_start = 1;);

  /* check if the upper bounds of the requested rows, if invalid */
  /* adjust                                                      */
  when(nrow < row_end,
       msg_message(2,"Requested rows are out of range\nChanging row_end to %d\n", nrow);
       row_end = nrow;);
  
  n = row_end - row_start + 1;

  fsize = fd_get_print_fsize(file);

  /* allocate memory */
  final = mem_malloc_safe(n*sizeof(char *));
  for (i = 0; i < n; i++) 
    final[i] = mem_malloc_safe(fsize*sizeof(char));


  /* fill character array final column by column */
  for (i = 0; i < fd_get_ncol(file); i++) {
    fd_print_col(file, i+1, final, row_start, n);
  }
  
  /* print to screen */
  for (i = 0; i < n; i++) 
    fprintf(output, "%s\n",final[i]);



  /* free memory */
  for (i = 0; i < n; i++) free(final[i]);
  free(final);


  /* handle flush/closing of file */
  fd_flush_close(file);

  return 0;
}


int fd_print_col(fd_file *file, int col, char **final, 
		 long row_start, long nrows) {

  if (fd_get_tabletype(file) == BINARY_TBL) {
    return fd_print_col_binary(file, col, final, row_start, nrows);
  } else if (fd_get_tabletype(file) == ASCII_TBL) {
    return fd_print_col_ascii(file, col, final, row_start, nrows);
  } else { 
    return -1;
  }
}

int fd_print_col_binary(fd_file *file, int col, char **final, 
			long row_start, long nrows) {
    
  char tmps[MAXSTRLEN] = "";
  char tmps2[MAXSTRLEN] = "";
  char  **tmpchr;
  short  *tmpsht;
  long   *tmplng;
  int    *tmpint;
  float  *tmpflt;
  double *tmpdbl;
  unsigned char *tmpbyt;
  int i, j;
  int anynull = 0;
  int status = 0;


  switch (file->typecode[col-1].int_type) {
  case TBIT: 
    msg_message(0,"fd_print_col: bit type not yet implemented\n");
    break;
  case TBYTE:
    if (file->typecode[col-1].vla) {
      tmpbyt = mem_malloc_safe(file->typecode[col-1].vla_length*
			       sizeof(unsigned char));

      for (j = 0; j < nrows; j++) {
	fits_read_col_byt(file->fptr, col, row_start + j, 
			  1, file->typecode[col-1].vla_length, 
			  0., tmpbyt, &anynull, &status);
	/* opening bracket */
	sprintf(tmps,"("); strcat(final[j],tmps); 
	for (i = 0; i < file->typecode[col-1].vla_length; i++) {
	  sprintf(tmps,"%3u", tmpbyt[i]); strcat(final[j],tmps);
	  /* comma's */
	  if (i < file->typecode[col-1].vla_length - 1)
	    strcat(final[j],",");
	} 
	/* closing bracket */
	sprintf(tmps,")  "); strcat(final[j],tmps); 
      }

      free(tmpbyt); 
      
    } else {
      tmpbyt = mem_malloc_safe(nrows*sizeof(unsigned char)); 
      
      fits_read_col_byt(file->fptr, col, row_start, 
			1, nrows, 0, tmpbyt, &anynull, &status);
      for (i = 0; i < nrows; i++) {
	sprintf(tmps,"%3u  ", tmpbyt[i]); strcat(final[i],tmps); 
      } 
    
    free(tmpbyt); 
    }
    break;
  case TLOGICAL:
    msg_message(0,"fd_print_col: logical type not yet implemented\n");
    break;
  case TSTRING:  
    if (file->typecode[col-1].vla) {
      tmpchr = mem_malloc_safe(file->typecode[col-1].vla_length*
			       sizeof(char *));
      for (i = 0; i < file->typecode[col-1].vla_length; i++) 
	tmpchr[i] = mem_malloc_safe(file->typecode[col-1].size*sizeof(char));
      
      for (j = 0; j < nrows; j++) {
	fits_read_col_str(file->fptr, col, row_start + j,
			  1, file->typecode[col-1].vla_length, 
			  "", tmpchr, &anynull, &status);
	/* opening bracket */
	sprintf(tmps,"("); strcat(final[j],tmps); 
	sprintf(tmps2,"%c%ds  ",'%',file->typecode[col-1].size);
	for (i = 0; i < file->typecode[col-1].vla_length; i++) {
	  sprintf(tmps, tmps2, tmpchr[i]); strcat(final[j],tmps); 
	  /* comma's */
	  if (i < file->typecode[col-1].vla_length - 1)
	    strcat(final[j],",");
	} 
	/* closing bracket */
	sprintf(tmps,")  "); strcat(final[j],tmps); 
      }

      for (i = 0; i < nrows; i++) free(tmpchr[i]);
      free(tmpchr); 
    } else {
      tmpchr = mem_malloc_safe(nrows*sizeof(char *));
      for (i = 0; i < nrows; i++) 
	tmpchr[i] = mem_malloc_safe(file->typecode[col-1].size*sizeof(char));
      
      fits_read_col_str(file->fptr, col, row_start, 
			1, nrows, "", tmpchr, &anynull, &status);
      sprintf(tmps2,"%c%ds  ",'%',file->typecode[col-1].size);
      for (i = 0; i < nrows; i++) {
	sprintf(tmps, tmps2, tmpchr[i]); strcat(final[i],tmps); 
      } 
      
      for (i = 0; i < nrows; i++) free(tmpchr[i]);
      free(tmpchr); 
    }

    break;
  case TSHORT:
    if (file->typecode[col-1].vla) {
      tmpsht = mem_malloc_safe(file->typecode[col-1].vla_length*sizeof(short));

      for (j = 0; j < nrows; j++) {
	fits_read_col_sht(file->fptr, col, row_start + j, 
			  1, file->typecode[col-1].vla_length, 
			  0., tmpsht, &anynull, &status);
	/* opening bracket */
	sprintf(tmps,"("); strcat(final[j],tmps); 
	for (i = 0; i < file->typecode[col-1].vla_length; i++) {
	  sprintf(tmps,"%6hd", tmpsht[i]); strcat(final[j],tmps);
	  /* comma's */
	  if (i < file->typecode[col-1].vla_length - 1)
	    strcat(final[j],",");
	} 
	/* closing bracket */
	sprintf(tmps,")  "); strcat(final[j],tmps); 
      }

      free(tmpsht); 
      
    } else {
      tmpsht = mem_malloc_safe(nrows*sizeof(short)); 
      
      fits_read_col_sht(file->fptr, col, row_start, 
			1, nrows, 0, tmpsht, &anynull, &status);
      for (i = 0; i < nrows; i++) {
	sprintf(tmps,"%6hd  ", tmpsht[i]); strcat(final[i],tmps); 
      } 
      
      free(tmpsht); 
    }
    break;
  case TINT:
    if (file->typecode[col-1].vla) {
      tmpint = mem_malloc_safe(file->typecode[col-1].vla_length*sizeof(int));

      for (j = 0; j < nrows; j++) {
	fits_read_col_int(file->fptr, col, row_start + j, 
			  1, file->typecode[col-1].vla_length, 
			  0., tmpint, &anynull, &status);
	/* opening bracket */
	sprintf(tmps,"("); strcat(final[j],tmps); 
	for (i = 0; i < file->typecode[col-1].vla_length; i++) {
	  sprintf(tmps,"%6d", tmpint[i]); strcat(final[j],tmps);
	  /* comma's */
	  if (i < file->typecode[col-1].vla_length - 1)
	    strcat(final[j],",");
	} 
	/* closing bracket */
	sprintf(tmps,")  "); strcat(final[j],tmps); 
      }

      free(tmpint); 
      
    } else {
      tmpint = mem_malloc_safe(nrows*sizeof(int)); 
      
      fits_read_col_int(file->fptr, col, row_start, 
			1, nrows, 0, tmpint, &anynull, &status);
      for (i = 0; i < nrows; i++) {
	sprintf(tmps,"%6d  ", tmpint[i]); strcat(final[i],tmps);
      } 
      
      free(tmpint); 
    }
    break;
  case TLONG:
    if (file->typecode[col-1].vla) {
      tmplng = mem_malloc_safe(file->typecode[col-1].vla_length*sizeof(long));

      for (j = 0; j < nrows; j++) {
	fits_read_col_lng(file->fptr, col, row_start + j, 
			  1, file->typecode[col-1].vla_length, 
			  0., tmplng, &anynull, &status);
	/* opening bracket */
	sprintf(tmps,"("); strcat(final[j],tmps); 
	for (i = 0; i < file->typecode[col-1].vla_length; i++) {
	  sprintf(tmps,"%11ld", tmplng[i]); strcat(final[j],tmps);
	  /* comma's */
	  if (i < file->typecode[col-1].vla_length - 1)
	    strcat(final[j],",");
	} 
	/* closing bracket */
	sprintf(tmps,")  "); strcat(final[j],tmps); 
      }

      free(tmplng); 
      
    } else {
      tmplng = mem_malloc_safe(nrows*sizeof(long)); 
      
      fits_read_col_lng(file->fptr, col, row_start, 
			1, nrows, 0, tmplng, &anynull, &status);
      for (i = 0; i < nrows; i++) {
	sprintf(tmps,"%11ld  ", tmplng[i]); strcat(final[i],tmps); 
      } 
      
      free(tmplng); 
    }

    break;
  case TFLOAT:
    if (file->typecode[col-1].vla) {
      tmpflt = mem_malloc_safe(file->typecode[col-1].vla_length*sizeof(float));

      for (j = 0; j < nrows; j++) {
	fits_read_col_flt(file->fptr, col, row_start + j, 
			  1, file->typecode[col-1].vla_length, 
			  0., tmpflt, &anynull, &status);
	/* opening bracket */
	sprintf(tmps,"("); strcat(final[j],tmps); 
	for (i = 0; i < file->typecode[col-1].vla_length; i++) {
	  sprintf(tmps,"%13.6e", tmpflt[i]); strcat(final[j],tmps);
	  /* comma's */
	  if (i < file->typecode[col-1].vla_length - 1)
	    strcat(final[j],",");
	} 
	/* closing bracket */
	sprintf(tmps,")  "); strcat(final[j],tmps); 
      }

      free(tmpflt); 
      
    } else {
      tmpflt = mem_malloc_safe(nrows*sizeof(float)); 
      
      fits_read_col_flt(file->fptr, col, row_start, 
			1, nrows, 0., tmpflt, &anynull, &status);
      for (i = 0; i < nrows; i++) {
	sprintf(tmps,"%13.6e  ", tmpflt[i]); strcat(final[i],tmps); 
      } 
      
      free(tmpflt); 
    }
    break;
  case TDOUBLE:
    if (file->typecode[col-1].vla) {
      tmpdbl = mem_malloc_safe(file->typecode[col-1].vla_length*
			       sizeof(double));

      for (j = 0; j < nrows; j++) {
	fits_read_col_dbl(file->fptr, col, row_start + j, 
			  1, file->typecode[col-1].vla_length, 
			  0., tmpdbl, &anynull, &status);
	/* opening bracket */
	sprintf(tmps,"("); strcat(final[j],tmps); 
	for (i = 0; i < file->typecode[col-1].vla_length; i++) {
	  sprintf(tmps,"%17.10e", tmpdbl[i]); strcat(final[j],tmps);
	  /* comma's */
	  if (i < file->typecode[col-1].vla_length - 1)
	    strcat(final[j],",");
	} 
	/* closing bracket */
	sprintf(tmps,")  "); strcat(final[j],tmps); 
      }

      free(tmpdbl); 
      
    } else {
      tmpdbl = mem_malloc_safe(nrows*sizeof(double)); 
      
      fits_read_col_dbl(file->fptr, col, row_start, 
			1, nrows, 0., tmpdbl, &anynull, &status);
      for (i = 0; i < nrows; i++) {
	sprintf(tmps,"%17.10e  ", tmpdbl[i]); strcat(final[i],tmps); 
      } 
      
      free(tmpdbl); 
    }
    break;
  case TCOMPLEX:
    msg_message(0,"fd_print_col: complex type not yet implemented\n");
    break;
  case TDBLCOMPLEX:
    msg_message(0,"fd_print_col: double complex type not yet implemented\n");
    break;
  default:
    msg_exit(2,"fd_print_col: unable to extract type information\n");
    break;
  }
  
  return 0;
  
}

int fd_print_col_ascii(fd_file *file, int col, char **final, 
		       long row_start, long nrows) {
    
  char tmps[MAXSTRLEN] = "";
  char format[MAXSTRLEN] = "";
  char  **tmpchr;
  int    *tmpint;
  float  *tmpflt;
  double *tmpdbl;
  int i;
  int anynull = 0;
  int status = 0;


  switch (file->typecode[col-1].int_type) {
  case TBIT: 
    msg_message(0,"fd_print_col_ascii: bit type not yet implemented\n");
    break;
  case TBYTE:
    break;
  case TLOGICAL:
    msg_message(0,"fd_print_col_ascii: logical type not yet implemented\n");
    break;
  case TSTRING:  
    tmpchr = mem_malloc_safe(nrows*sizeof(char *));
    for (i = 0; i < nrows; i++) 
      tmpchr[i] = mem_malloc_safe(file->typecode[col-1].size*sizeof(char));

    fits_read_col_str(file->fptr, col, row_start, 
		      1, nrows, "", tmpchr, &anynull, &status);
    sprintf(format,"%c%ds  ",'%',file->typecode[col-1].size);
    for (i = 0; i < nrows; i++) {
      sprintf(tmps,format, tmpchr[i]); strcat(final[i],tmps); 
    } 

    for (i = 0; i < nrows; i++) free(tmpchr[i]);
    free(tmpchr); 
    break;
  case TSHORT:
    msg_message(0,"fd_print_col_ascii: short type not yet implemented\n");
    break;
  case TINT:
    tmpint = mem_malloc_safe(nrows*sizeof(int)); 

    fits_read_col_int(file->fptr, col, row_start, 
		      1, nrows, 0, tmpint, &anynull, &status);
    for (i = 0; i < nrows; i++) {
      sprintf(tmps,"%6d  ", tmpint[i]); strcat(final[i],tmps);
    } 

    free(tmpint); 
    break;
  case TLONG:
    msg_message(0,"fd_print_col_ascii: long type not yet implemented\n");
    break;
  case TFLOAT:
    tmpflt = mem_malloc_safe(nrows*sizeof(float)); 

    fits_read_col_flt(file->fptr, col, row_start, 
		      1, nrows, 0., tmpflt, &anynull, &status);
    if (file->typecode[col-1].type == 'F') {
      sprintf(format,"%c%d.%df  ",'%',
	      file->typecode[col-1].size,file->typecode[col-1].precision);
    } else if (file->typecode[col-1].type == 'E') {
      sprintf(format,"%c%d.%de  ",'%',
	      file->typecode[col-1].size,file->typecode[col-1].precision);
    } else {
      msg_message(0,"Error (fd_print_col_ascii): unknown format type for float. Use %cf instead",'%');
      sprintf(format,"%cf  ",'%');
    }
    for (i = 0; i < nrows; i++) {
      sprintf(tmps, format, tmpflt[i]); strcat(final[i],tmps); 
    } 

    free(tmpflt); 
    break;
  case TDOUBLE:
    tmpdbl = mem_malloc_safe(nrows*sizeof(double)); 

    fits_read_col_dbl(file->fptr, col, row_start, 
		      1, nrows, 0., tmpdbl, &anynull, &status);
    sprintf(format,"%c%d.%de  ", '%',
	    file->typecode[col-1].size,file->typecode[col-1].precision);
    for (i = 0; i < nrows; i++) {
      sprintf(tmps, format, tmpdbl[i]); strcat(final[i],tmps); 
    } 

    free(tmpdbl); 
    break;
  case TCOMPLEX:
    msg_message(0,"fd_print_col_ascii: complex type not yet implemented\n");
    break;
  case TDBLCOMPLEX:
    msg_message(0,"fd_print_col_ascii: double complex type not yet implemented\n");
    break;
  default:
    msg_exit(2,"fd_print_col_ascii: unable to extract type information\n");
    break;
  }
  
  return 0;
  
}


int fd_get_print_fsize(fd_file *file) {

  int i;
  int fsize = 0;
  int tabletype = fd_get_tabletype(file);

  for (i = 0; i < fd_get_ncol(file); i++) {
    switch (file->typecode[i].int_type) {
    case TBIT: 
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ? 
	  file->typecode[i].vla_length * (file->typecode[i].size + 1) + 1 : 
	  file->typecode[i].size;
      else 
	msg_message(2,"TBIT is not a valid type specifier for ASCII_TBL");
      break;
    case TBYTE:          /* %3u */
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ?
	  file->typecode[i].vla_length * (3*file->typecode[i].size + 1) + 1 : 
	  3*file->typecode[i].size; 
      else 
	msg_message(2,"TBYTE is not a valid type specifier for ASCII_TBL");
      break;
    case TLOGICAL:        
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ?
	  file->typecode[i].vla_length * (1 + 1) + 1 :
	  1;
      else 
	msg_message(2,"TLOGICAL is not a valid type specifier for ASCII_TBL");
      break;
    case TSTRING:         /* %s (size obtained from the typecode */
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ?
	  file->typecode[i].vla_length * (file->typecode[i].size + 1) + 1 :
	  file->typecode[i].size;
      else 
	fsize += file->typecode[i].size;
      break;
    case TSHORT:          /* %6hd */
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ?
	  file->typecode[i].vla_length * (6 + 1) + 1 :
	  6;
      else 
	msg_message(2,"TSHORT is not a valid type specifier for ASCII_TBL");
      break;
    case TINT:            /* %6d */
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ?
	  file->typecode[i].vla_length * (6 + 1) + 1 :
	  6;
      else 
	fsize += file->typecode[i].size;
      break;
    case TLONG:           /* %10ld */
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ?
	  file->typecode[i].vla_length * (11 + 1) + 1 :
	  11;
      else 
	msg_message(2,"TLONG is not a valid type specifier for ASCII_TBL");
      break;
    case TFLOAT:          /* %11.6e */
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ?
	  file->typecode[i].vla_length * (13 + 1) + 1 :
	  13;
      else 
	fsize += file->typecode[i].size;
      break;
    case TDOUBLE:         /* %17.10e */
      if (tabletype == BINARY_TBL)
	fsize += (file->typecode[i].vla) ?
	  file->typecode[i].vla_length * (17 + 1) + 1 :
	  17;
      else 
	fsize += file->typecode[i].size;
      break;
    case TCOMPLEX:
      msg_message(0,"fd_get_fsize: complex type not yet implemented\n");
      break;
    case TDBLCOMPLEX:
      msg_message(0,"fd_get_fsize: double complex type not yet implemented\n");
      break;
    default:
      msg_exit(2,"fd_get_fsize: unable to extract type information\n");
      break;
    }
    fsize += 2;
    
  }
  
  fsize += 1;         /* to account for the trailing '\0' */

  return fsize;

}
