#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "messages.h" 

double exint_n(double x, double E1x_in, int n) {

/* Calculates E_n(x), for n = 1,2,3,4. If n>1, and E1x_in >= 0, then 
   routine uses E1x for E_1(x) in calculating recurrance.
   Returns a double precision floating point value. */

  double E1x;
  double x2, x3, x4;

  static double a[] = {
    7.122452e-07, 1.766345e-06, 2.928433e-05, .0002335379, .001664156,
    .01041576, .05555682, .2500001, .9999999, .57721566490153};
  static double b[] = {
    8.5733287401, 18.059016973, 8.6347608925, .2677737343, 9.5733223454,
    25.6329561486, 21.0996530827, 3.9584969228};
  
  E1x = E1x_in;

  if ((n==1)||(E1x < 0)) {
    if (x==0.) {
      return(-1.);
    } else {
      if (x <= 1.0) {
	E1x = ((((((((a[0]*x-a[1])*x+a[2])*x-a[3])*x+a[4])*x-a[5])*x+a[6])*x
		 -a[7])*x+a[8])*x - log(x) - a[9];
      } else {
	x2 = x*x;
	x3 = x2*x;
	x4 = x2*x2;
	E1x = (x4+b[0]*x3+b[1]*x2+b[2]*x+b[3])/
	  (x4+b[4]*x3+b[5]*x2+b[6]*x+b[7]);
	E1x = E1x / (x * exp(x));
      }
    }
  }
  
  switch (n) {
  case (1):
    return (E1x);
    break;
  case (2):
    return (exp(-x) - x*E1x);
    break;
  case (3):
    return (x*x*E1x + exp(-x)*(1-x))/2.;
    break;
  case (4):
    return (exp(-x)*(2-x+x*x) - x*x*x*E1x)/6.;
    break;
  default:
    errmess("exint_n","Bad value for n = %d",n);
    return -1;
    break;
  }

  return -1.;
}
