#include <stdlib.h>
#include <stdio.h>

#include "apec.h" 
#include "messages.h"

void write_debug(struct DEBUG_GENERIC *debug, int ascii, int fits, 
		 char* fmt, ...);

void calc_emission_features(struct DEBUG_FILES *debug, 
			    struct ELEMENT_ION_STATE *state,
			    double abund, double Te, double Ion_pop, 
			    struct EMISSION_LINES **line, 
			    struct BIN_SPEC *spectrum) {
  
  /* Goal here is to go through the "emission features" list and add the
     new lines into the line list, with the appropriate power. */

  int rmJ, Z;
  int ll, lu;
  long max_emiss, iLine;

  double lambda, lambda_err, epsilon, Plambda;
  double kT;

  struct EMISSION_LINES *lptr;
  
  kT = KBOLTZ*Te;
  Z = state->ion_inf->Z;
  rmJ = state->ion_inf->rmJ;
  max_emiss = state->ion_inf->nemiss;

  lptr = *line;

  /* Calculate the line emission  */

  for (iLine = 0; iLine < max_emiss; iLine++) {
    epsilon = 0.;
    lambda = state->emission_dat[iLine].lambda;
    lambda_err = state->emission_dat[iLine].lambda_err;
    ll = state->emission_dat[iLine].lev_lower;
    lu = state->emission_dat[iLine].lev_upper;
    switch(state->emission_dat[iLine].type) {
    case (0):
      epsilon = 0.0;
      break;
    case (1):
      epsilon = 0.0;
      break;
    default:
      errmess("calc_emission_features","Unknown emission feature type %d",
	      state->emission_dat[iLine].type);
      break;
    }

    if (debug != NULL) {
      if (debug->do_satel) {
	write_debug(&debug->satel, debug->ascii, debug->fits,
		    "%3d %3d %5d%5d %10g %10g %10g %10g %10g\n",
		    Z, rmJ, lu, ll, Te, lambda, 0.0, 0.0, epsilon );
      }
    }

    if (lambda > 0) {
      if (lptr == NULL) { 
	*line = (struct EMISSION_LINES *)
	  malloc_safe(sizeof(struct EMISSION_LINES));
	lptr = *line;
      } else { /* Find last link & make new one */
	while (lptr->next != NULL) lptr = lptr->next;
	lptr->next = (struct EMISSION_LINES *)
	  malloc_safe(sizeof(struct EMISSION_LINES));
	lptr = lptr->next;
      }
      lptr->next = NULL;

      Plambda = epsilon * (HC_IN_ERG_A/lambda);  /*  erg cm^3 / s */
    
      lptr->epsilon = epsilon; /* phot cm^3 / s */
      lptr->Plambda = Plambda;  /*  erg cm^3 / s */
      lptr->lambda  = lambda;  /* Angstroms */
      lptr->lambda_err=lambda_err;  /* Angstroms */
      lptr->Z = Z;
      lptr->rmJ = rmJ;
      lptr->ul= lu;
      lptr->ll= ll;
      lptr->type = EMISSION_FEATURE_LINE;
      if (debug != NULL) {
	if ((debug->do_emiss)&&(epsilon != 0)) {
	  write_debug(&debug->emiss, debug->ascii, debug->fits,
		      "%2d %2d %7.2f %7.2f %2d %2d %8.2f %8.2e %8.2e\n",
		      Z,rmJ,-10.0,log10(Te),lu,ll,lambda,epsilon,Plambda);
	}
      }
    } else {
      /* message("calc_emission_features","Bad line wavelength") */
    }
  }
  if (debug != NULL) {
    if (debug->do_satel && debug->ascii) fflush(debug->satel.ascii);
    if (debug->do_emiss && debug->ascii) fflush(debug->emiss.ascii);
  }
}
