#include <stdlib.h>
#include <stdio.h>

#include "apec.h" 
#include "messages.h"

void calc_spectrum_per_ion(int Z, double Ion_pop[], 
			   struct EMISSION_LINES *line, 
			   struct BIN_SPEC *spectrum) {
  /* Divide out ion population from emission lines and from continuum 
     spectrum */

  int iBin;
  struct EMISSION_LINES *ptr;
  struct COMPRESS_CONT *csptr;

  /* Begin by dividing it out of all emission lines */
  ptr = line;
  /* message("calc_spectrum_per_ion","Z = %d",Z); */
  while (ptr != NULL) {
    if (ptr->Z == Z) {
      switch (ptr->type) {
      case (BOUNDBOUND_LINE):
	ptr->epsilon = ptr->epsilon/Ion_pop[ptr->rmJ-1];
	ptr->Plambda = ptr->Plambda/Ion_pop[ptr->rmJ-1];
	break;
      case (SATELLITE_LINE):
	ptr->epsilon = ptr->epsilon/Ion_pop[ptr->rmJ-1];
	ptr->Plambda = ptr->Plambda/Ion_pop[ptr->rmJ-1];
	break;
      case (IONEXC_LINE):
	ptr->epsilon = ptr->epsilon/Ion_pop[ptr->rmJ-2];
	ptr->Plambda = ptr->Plambda/Ion_pop[ptr->rmJ-2];
	break;
      default:
	errmess("calc_spectrum_per_ion","Line type %d not recognized",
		ptr->type);
      }
    }
    ptr = ptr->next;
  }

  /* Remove the total emission values, meaningless in this case */
  spectrum->total_emission[Z] = 0.0;
  spectrum->tot_line_emis[Z]  = 0.0;  
  spectrum->tot_brems_emis[Z] = 0.0;
  spectrum->tot_rrc_emis[Z]   = 0.0;  
  spectrum->tot_twoph_emis[Z] = 0.0;

  /* Now zero the summed continuum bins, which are also meaningless here */
  for (iBin=0;iBin<spectrum->Nbin;iBin++) {
    (spectrum->bin_emission[Z])[iBin] = 0.0; 
    (spectrum->pseudo_emiss[Z])[iBin] = 0.0; 
    (spectrum->brems_emiss[Z])[iBin]  = 0.0;  
    (spectrum->rrc_emiss[Z])[iBin]    = 0.0;    
    (spectrum->twoph_emiss[Z])[iBin]  = 0.0;  
    (spectrum->bin_error[Z])[iBin]    = 0.0;  
  }

  /* Now fix the per-ion continuum spectrum, which _does_ have meaning. */
  csptr = spectrum->cs;
  while (csptr != NULL) {
    if (csptr->Z == Z) {
      if (csptr->brems != NULL) {
	for (iBin=0;iBin<csptr->Nbrems;iBin++) 
	  csptr->brems[iBin] = csptr->brems[iBin]/Ion_pop[csptr->rmJ-1];
      }
      if (csptr->twoph != NULL) {
	for (iBin=0;iBin<csptr->Ntwoph;iBin++) 
	  csptr->twoph[iBin] = csptr->twoph[iBin]/Ion_pop[csptr->rmJ-1];
      }
      if (csptr->rrc != NULL) {
	for (iBin=0;iBin<csptr->Nrrc;iBin++) 
	  csptr->rrc[iBin] = csptr->rrc[iBin]/Ion_pop[csptr->rmJ-1];	
      }
      if (csptr->real != NULL) {
	for (iBin=0;iBin<csptr->Nreal;iBin++) 
	  csptr->real[iBin] = csptr->real[iBin]/Ion_pop[csptr->rmJ-1];
      }
      if (csptr->pseudo != NULL) {
	for (iBin=0;iBin<csptr->Npseudo;iBin++) 
	  csptr->pseudo[iBin] = csptr->pseudo[iBin]/Ion_pop[csptr->rmJ-1];
      }
      if (csptr->bin_real != NULL) {
	for (iBin=0;iBin<spectrum->Nbin;iBin++) 
	  csptr->bin_real[iBin] = csptr->bin_real[iBin]/Ion_pop[csptr->rmJ-1];
      }
    }
    csptr = csptr->next;
  }
}
