#include <stdlib.h>
#include <stdio.h>
#include <fitsio.h>

#include "messages.h"

void fits_printerror(char *process, int status);

unsigned long check_fits_file(fitsfile *fptr) {

  int dataok=0, hduok=0;
  int status=0;
  char filename[1024];
  unsigned long datasum=0, hdusum=0;

  if (fits_verify_chksum(fptr, &dataok, &hduok, &status)) {
    fits_printerror("check_fits_file", status);
  }

  if (fits_file_name(fptr, filename, &status)) {
    fits_printerror("check_fits_file", status);
  }

  if (dataok==0) {
    message("check_fits_file","DATASUM not present in file %s",filename);
    return 0;
  }

  if (hduok==0) {
    message("check_fits_file","CHECKSUM not present in file %s",filename);
    return 0;
  }
  
  if (dataok==-1) {
    errmess("check_fits_file",
	    "Data hecksum (DATASUM) mismatch when reading file %s",filename);
  }

  if ((dataok==1)&&(hduok==-1)) {
    message("check_fits_file",
	    "Header checksum (CHECKSUM) mismatch when reading file %s",
	    filename);
    message("check_fits_file","Data is OK, but file should be checked.");
  }
  
  if (fits_get_chksum(fptr, &datasum, &hdusum, &status)) {
    fits_printerror("check_fits_file", status);
  }
  
  return datasum;
}
