#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "messages.h"
#include "apec.h"
#include "datatypes.h"

void expand_env(char *string);

void load_filemap(char *filemap_file, struct FILEMAP **filemap) {

  FILE *fptr;
  char line[MAXSTRLEN];
  int linenum=0;
  struct FILEMAP *current, *previous=NULL;
  struct stat buf;

  fptr = fopen(filemap_file,"r");
  if (fptr == NULL) {
    errmess("load_filemap","Cannot open filemap file %s",filemap_file);
  }

  *filemap = (struct FILEMAP *) malloc_safe(sizeof(struct FILEMAP));
  current = *filemap;

  while (fgets(line,MAXSTRLEN-1,fptr) != NULL) {
    linenum++;
    if (sscanf(line,"%d %d %d %s\n", &(current->datatype), &(current->Z),
	       &(current->rmJ), current->filename) != 4) {
      message("load_filemap","Filemap (%s) line (%d) not understood: %s",
	      filemap_file, linenum, line);
    } else {

      /* Now, at this point, filename could be a fully-qualified filename.*/
      /* Or it could have an environment variable in it. */
      /* We need to distinguish between these, and do some checks. */

      expand_env(current->filename);

      /* Check to see if file exists */
      if (stat(current->filename, &buf)!=0) { 
	message("load_filemap","Error with file %s",current->filename);
	perror("Cannot stat file");
      } else {
	/* message("load_filemap","Got file %s",current->filename); */
      }

      current->next = (struct FILEMAP *) malloc_safe(sizeof(struct FILEMAP));
      previous = current;
      current = current->next;
    } 
  }
  previous->next = NULL;  /* Terminate correctly */
  
  /* And close the file... */
  if (fclose(fptr) != 0) {
    message("load_filemap","Error closing filemap file");
  }
}
