#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <fitsio.h>

#include "messages.h"
#include "apec.h"
#include "datatypes.h"

int moveto_ion_hdu(fitsfile *fptr, int Z, int rmJ,int datatype,char *ion_name);
char *roman_element(int Z, int rmJ);

int get_filename_from_map(struct FILEMAP *filemap, int Z, int rmJ,
			  int datatype, char **filename);

void open_file_filemap(fitsfile **fptr, int Z, int rmJ, int datatype, 
		       char *ion_name, struct FILEMAP *filemap,
		       char **filename) {

  int status=0;

  if (get_filename_from_map(filemap, Z, rmJ, datatype, filename)) {
    /* ************************************************************** */
    /* message("open_filemap","No filename for %s, type %d, skipped", */
    /* roman_element(Z,rmJ-1), datatype);                             */
    /* ************************************************************** */
    *fptr = NULL;
    return;
  }

  if (fits_open_file(fptr, *filename, READONLY, &status)) {
    fitsmess("open_file_filemap",status,"Error opening file %s.",*filename);
  }

  if (moveto_ion_hdu(*fptr, Z, rmJ, datatype, ion_name)==0) {
    message("open_file_filemap","No data of type %d found for ion %s, skipped",
	    datatype, ion_name);
    if (fits_close_file(*fptr, &status)) { /* Close the file */
      fitsmess("open_file_filemap",status,"Error closing file %s.",*filename);
    }
    *fptr = NULL;  /* Return a NULL pointer */
  }
}

/* Given a filemap, a Z, rmJ pair, and a datatype, find the appropriate */
/* filename */

int get_filename_from_map(struct FILEMAP *filemap, int Z, int rmJ,
			  int datatype, char **filename) {

  int done, found, retval;
  char *def_filename;

  done = FALSE;
  found = FALSE;
  def_filename = NULL;
  *filename = NULL;

  while (!done) {
    if ((filemap->Z==Z)&&(filemap->datatype==datatype)) { 
      if (filemap->rmJ==rmJ) {
	done = TRUE;
	found = TRUE;
	*filename = filemap->filename;
      }
      if (filemap->rmJ == -1) {
	if (def_filename==NULL) {
	  def_filename = filemap->filename;  /* Default */
	  found = TRUE; /* Well, at least a default was found. */
	} else {
	  errmess("open_file_filemap","Too many default files in filemap!");
	}
      }
    }
    filemap=filemap->next;
    if (filemap==NULL) done=TRUE;
  }
  if ((*filename==NULL)&&(def_filename!=NULL)) *filename = def_filename;

  if (found) { retval = 0; } else { retval = 1; }

  return retval;
}

