#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <fitsio.h>

#include "apec.h"
#include "messages.h"

void fits_spec_write_cocoHDU(int N_Z, int list_Z[], int list_rmJ[], 
			     fitsfile *fptr, 
			     double T, double Ne, double tot_emission, 
			     double **E_real, double **E_pseudo, 
			     double **real_cont, double **pseudo_cont,
			     int *Ncomp_real, int *Ncomp_pseudo);
void compress_energy_spectrum(int Nbin, double *bin_edge, double *emission, 
			      double error, char *type,
			      int *Nval, double **Eval, double **Fluxval);
void compress_photon_spectrum(double *E, double *flux, int *Nbin,
			      double error);

void output_comp_emission(struct PARAMETERS *params, double T, double Ne, 
			  struct BIN_SPEC *spectrum) {

  int iZ, irmJ, rmJ, Z=0, N_Z, N_I=0, iI;
  double tot_emission;  /* in erg cm^3 s^-1 */
  double *zeros;
  int nbin, iBin;
  int *Zlist, *rmJlist;
  struct COMPRESS_CONT *csptr;

  /* Continuum related */
  double **E_real, **E_pseudo, **real_cont, **pseudo_cont;
  int *Ncomp_real, *Ncomp_pseudo;

  N_Z = params->N_Elements;
  tot_emission = 0.0;

  /* Count how many spectra we've got here, and get the total emission  */
  N_I = 1 + N_Z;  /* 1 for total spectrum, plus N_Z for each element */
  for (iZ=0;iZ<N_Z;iZ++) N_I += params->list_NrmJ[iZ];

  nbin = (int) spectrum->Nbin;

  E_real      = (double **) malloc_safe(N_I*sizeof(double *));
  E_pseudo    = (double **) malloc_safe(N_I*sizeof(double *));
  real_cont   = (double **) malloc_safe(N_I*sizeof(double *));
  pseudo_cont = (double **) malloc_safe(N_I*sizeof(double *));

  Ncomp_real    = (int *) malloc_safe(N_I*sizeof(int));
  Ncomp_pseudo  = (int *) malloc_safe(N_I*sizeof(int));
  Zlist         = (int *) malloc_safe(N_I*sizeof(int));
  rmJlist       = (int *) malloc_safe(N_I*sizeof(int));

  /* Now we need to lay out all the spectra.  First is the total summed */
  /* spectra, then each one of the elements (total) followed by its ions */

  iI = 0;  /* iI = 0 is the total summed emission from all elements */
  /* Set up all the basic values for the total summed emission */
  Zlist[iI] = 0;
  rmJlist[iI] = 0;
  E_real[iI]      = (double *) malloc_safe((nbin+1)*sizeof(double));
  E_pseudo[iI]    = (double *) malloc_safe((nbin+1)*sizeof(double));
  real_cont[iI]   = (double *) malloc_safe(nbin*sizeof(double));
  pseudo_cont[iI] = (double *) malloc_safe(nbin*sizeof(double));
  Ncomp_real[iI] = nbin;
  Ncomp_pseudo[iI] = nbin;
  (E_real[iI])[0]  = spectrum->bin_edge[0];
  (E_pseudo[iI])[0]= spectrum->bin_edge[0];
  for (iBin=0;iBin<nbin;iBin++) {
      (E_real[iI])[iBin+1]  = spectrum->bin_edge[iBin+1];
      (E_pseudo[iI])[iBin+1]= spectrum->bin_edge[iBin+1];
      (real_cont[iI])[iBin]  = 0.0;
      (pseudo_cont[iI])[iBin]= 0.0;
  } 
   
  for (iZ=0;iZ<N_Z;iZ++) {
    iI++;
    Z = params->list_Z[iZ];
    Zlist[iI] = Z;
    rmJlist[iI] = 0;  /* Summed */
    E_real[iI]      = (double *) malloc_safe((nbin+1)*sizeof(double));
    E_pseudo[iI]    = (double *) malloc_safe((nbin+1)*sizeof(double));
    real_cont[iI]   = (double *) malloc_safe(nbin*sizeof(double));
    pseudo_cont[iI] = (double *) malloc_safe(nbin*sizeof(double));
    
    /* Now start summing up data, total and per element */
    Ncomp_real[iI] = nbin;
    Ncomp_pseudo[iI] = nbin;
    (E_real[iI])[0]   = spectrum->bin_edge[0];
    (E_pseudo[iI])[0] = spectrum->bin_edge[0];
    for (iBin=0;iBin<nbin;iBin++) {  /* Go through each energy */
      (E_real[iI])[iBin+1]  = spectrum->bin_edge[iBin+1];
      (E_pseudo[iI])[iBin+1]= spectrum->bin_edge[iBin+1];
      /* Put the per element values in our arrays*/
      (real_cont[iI])[iBin] = (spectrum->brems_emiss[Z])[iBin] +
	(spectrum->rrc_emiss[Z])[iBin] + (spectrum->twoph_emiss[Z])[iBin];
      (pseudo_cont[iI])[iBin] = (spectrum->pseudo_emiss[Z])[iBin];
      /* Now sum up the totals */
      (real_cont[0])[iBin]   += (real_cont[iI])[iBin];
      (pseudo_cont[0])[iBin] += (pseudo_cont[iI])[iBin];
    }
    
    /* Now, compress the per element data, both real and pseudo */
    compress_photon_spectrum(E_real[iI], real_cont[iI], &(Ncomp_real[iI]),
			     ERROR_FINAL);
    compress_photon_spectrum(E_pseudo[iI], pseudo_cont[iI],
			     &(Ncomp_pseudo[iI]), ERROR_FINAL);

  }
  
  for (iZ=0;iZ<N_Z;iZ++) {
    Z = params->list_Z[iZ];
    /* Now copy all the compressed per ion spectra */
    for (irmJ=0;irmJ<params->list_NrmJ[iZ];irmJ++) {
      rmJ = params->list_rmJ[iZ][irmJ];
      csptr = spectrum->cs;
      while ((csptr != NULL) && ((csptr->Z != Z) || (csptr->rmJ != rmJ))) {
	csptr = csptr->next;
      }
      
      if ((csptr != NULL)&&(csptr->Nbrems > 0 || csptr->Ntwoph > 0 ||
			    csptr->Nrrc > 0   || csptr->Npseudo > 0 )) {
	iI++;
	Zlist[iI] = Z;
	rmJlist[iI] = rmJ;
	if (csptr->Npseudo == 0) { /* There are no lines! */
	  zeros = malloc(nbin*sizeof(double));
	  for (iBin=0;iBin<nbin;iBin++) zeros[iBin] = 0.0;
	  compress_energy_spectrum(spectrum->Nbin, spectrum->bin_edge, 
				   zeros, ERROR_FINAL, 
				   "Pseudo Continuum", &(csptr->Npseudo), 
				   &(csptr->E_pseudo), &(csptr->pseudo));
	  free(zeros);
	}
	E_pseudo[iI] = csptr->E_pseudo;
	pseudo_cont[iI] = csptr->pseudo;
	Ncomp_pseudo[iI] = csptr->Npseudo;

	/* bin_real is the sum of brems + two_phot + rrc */
	compress_energy_spectrum(spectrum->Nbin, spectrum->bin_edge, 
				 csptr->bin_real, ERROR_FINAL, 
				 "Real Continuum", &(csptr->Nreal), 
				 &(csptr->E_real), &(csptr->real));
	E_real[iI] = csptr->E_real; 
	real_cont[iI] = csptr->real;
	Ncomp_real[iI] = csptr->Nreal;
      }
    }
  }

  /* And now compress the final sum */
  compress_photon_spectrum(E_real[0], real_cont[0], &(Ncomp_real[0]),
			   ERROR_FINAL);
  compress_photon_spectrum(E_pseudo[0], pseudo_cont[0],&(Ncomp_pseudo[0]),
			   ERROR_FINAL);

  /* Now time to do the output */
  if (iI + 1 != N_I ) 
    /* message("output_comp_emission","Max ions: %d; actual %d",N_I, iI); */

  fits_spec_write_cocoHDU(iI, Zlist, rmJlist, params->fptr_comp, T, Ne, 
			  tot_emission, E_real, E_pseudo, real_cont, 
			  pseudo_cont, Ncomp_real, Ncomp_pseudo);
  
  /*message("output_total_emission","T = %e, Emission = %e",T, 
    tot_emission);*/

  for (iZ=0;iZ<N_Z+1;iZ++) {
    free(E_real[iZ]);
    free(E_pseudo[iZ]);
    free(real_cont[iZ]);
    free(pseudo_cont[iZ]);
  }
  free(Zlist);
  free(rmJlist);
  free(Ncomp_real);
  free(Ncomp_pseudo);
  free(E_real);
  free(E_pseudo);
  free(real_cont);
  free(pseudo_cont);
  
}

