FUNCTION calc_chianti, data, dE, T
;
;       DESCALE2_UPS,xt,c,t_type,spline_values,upsilon
; INPUTS:
;	xt:  scaled temperature
;        c:  Burgess-Tully scaling parameter
;       t_type:  B-T transition type (1-4)
;       spline_values:  spline values for B-T scaled upsilons	
;	see Burgess and Tully, 1992, Astron and Astrophys, 254, 436.
;   to rescale the energies and upsilons
;   (thermal collision strengths)
;   modified version of descale_ups which uses data in common
;
;  xt=kt/de
;
;
;
KBOLTZ = 8.617385e-5  ; ev/K
xt = KBOLTZ*T/dE    
c = data.coeff_om(0)
t_type = data.coeff_type - 10
spl = data.coeff_om(1:5)

CASE t_type OF

1:  begin
      st=1.-alog(c)/alog(xt+c)
      xs=findgen(5)*0.25
      y2=nr_spline(xs,spl)
      sups=nr_splint(xs,spl,y2,st)
      ups=sups*alog(xt+exp(1.))
    end
;
2:  begin
      st=xt/(xt+c)
      xs=findgen(5)*0.25
      y2=nr_spline(xs,spl)
      sups=nr_splint(xs,spl,y2,st)
      ups=sups
    end
;
3:  begin
      st=xt/(xt+c)
      xs=findgen(5)*0.25
      y2=nr_spline(xs,spl)
      sups=nr_splint(xs,spl,y2,st)
      ups=sups/(xt+1.)
;
    end
;
4:  begin
      st=1.-alog(c)/alog(xt+c)
      xs=findgen(5)*0.25
      y2=nr_spline(xs,spl)
      sups=nr_splint(xs,spl,y2,st)
      ups=sups*alog(xt+c)
   end
;
else:  print,' t_type ne 1,2,3,4=',t_type(indx,jndx),indx,jndx
;
ENDCASE
;
ups=ups>0.
;
if((max(st) gt 1.) or (min(st) lt 0.)) then $
  print,xt,st, ' st outside 0. to 1. range'

return, ups

end

