FUNCTION calc_gamma, Z, rmJ, ll, lu, T, SILENT=silent
;
;  data is an Ecoll_S structure, Z is the element, rmJ the roman
;  number of the ion, ll the lower level, lu the upper level, and T
;  the temperature, which may be a vector.
;
if (n_elements(silent) eq 0) then silent=0
;
; Is data an Ecoll_S ?
;
structtype = tag_names(data,/struct)
if ((strpos(structtype,'ECOLL_S10') eq -1) and $
    (strpos(structtype,'ECOLL_S11') eq -1)) then begin
    print,strcompress('data is incorrect type ('+structtype+$
                      '); should be EColl_S10 or EColl_S11')
    return,0*T
endif
;
; Now, does the appropriate ll, lu exist in this data structure?
;
gp = where((data.lower_lev eq ll)and(data.upper_lev eq lu),count)
if (count eq 0) then begin
    if (silent eq 1) then print,strcompress('No such transition '+ll+'->'+lu+$
                                            ' in the data structure')
    return,0*T
endif
if (count gt 1) then begin
    if (silent eq 1) then print,strcompress('More than one such transition '+$
                                            ll+'->'+lu+$
                                            ' in the data structure!')
    return,0*T
endif
;
; OK, at this point, we've got a single good transition
;
tr = data(gp)
;
; Now, check the transition temperatures
;
gp = where((T ge tr.min_temp) and (T le tr.max_temp),count_gp)
bp = where((T lt tr.min_temp) and (T gt tr.max_temp),count_bp)
if (count_gp eq 0) then begin
    print,'No temperatures with allowed range for transition.'
    return,0*T  
endif

result = fltarr(n_elements(T))
if (count_bp gt 0) then result(bp) = 0.0 ; set the out-of-range pts to zero

if ((tr.Coeff_Type ge 100)and (tr.Coeff_Type le 199)) then begin 
    result(gp) = interpol(tr.Coeff_OM,tr.Temperature,T(gp))
    if (silent ne 1) then print,'calculated E upsilon'
endif
if ((tr.Coeff_Type ge 200)and (tr.Coeff_Type le 299)) then begin 
    result(gp) = interpol(tr.Coeff_OM,tr.Temperature,T(gp))
    if (silent ne 1) then print,'calculated P upsilon'
endif
if ((tr.Coeff_Type ge 300)and (tr.Coeff_Type le 399)) then begin 
    result(gp) = interpol(tr.Coeff_OM,tr.Temperature,T(gp))
    if (silent ne 1) then print,'calculated E rate coefficient'
endif
if ((tr.Coeff_Type ge 400)and (tr.Coeff_Type le 499)) then begin 
    result(gp) = interpol(tr.Coeff_OM,tr.Temperature,T(gp))
    if (silent ne 1) then print,'calculated P rate coefficient'
endif
if ((tr.Coeff_Type gt 10)and(tr.Coeff_Type lt 15)) then begin ; Chianti
    c = tr.Coeff_OM[0]
    if (collision->coll_type == CHIANTI_1) st = 1 - (log(c)/log(chi_inv+c)) ;
    if (collision->coll_type == CHIANTI_2) st = chi_inv/(chi_inv+c);
    if (collision->coll_type == CHIANTI_3) st = chi_inv/(chi_inv+c);
    if (collision->coll_type == CHIANTI_4) st = 1 - (log(c)/log(chi_inv+c));
    

endif
if ((tr.Coeff_Type gt 30)and(tr.Coeff_Type lt 34)) then begin ; SGC

endif
if ((tr.Coeff_Type eq 41)or(tr.Coeff_Type eq 42)) then begin ; Kato
    calc_kato
endif
if (tr.Coeff_Type eq 1) then begin ; Burgess&Tully

endif

return, result
end
