PRO zscale_wgfa, Z, rmJ, outfile, ZARR=Zarr
;
;  First, get values to scale from
;
if (n_elements(Zarr) eq 0) then Zarr = [Z-1,Z+1]
rmJarr = Zarr - (Z - rmJ)
;
;  Now, load these files.
;
read_eline, Z-1, rmJ-1, dataM1, headerM1, FOUND=found,/silent
if (found eq 0) then begin
    print,strcompress('Cannot find required ion '+roman_element(Z-1,rmJ-1))
    return
endif
;
;  Assume the ion (minus 1 proton) is a good estimate for everything
;  except the wavelength and the A value.
;
nanvalue = -999.0
Ntr = n_elements(dataM1)
scale_data = dataM1
for iTr=0,Ntr-1 do scale_data(iTr).wavelen = 0.0
for iTr=0,Ntr-1 do scale_data(iTr).wave_obs = nanvalue
for iTr=0,Ntr-1 do scale_data(iTr).Einstein_A = 0.0
for iTr=0,Ntr-1 do scale_data(iTr).wave_ref = '2000APED.Z-scale..A'
for iTr=0,Ntr-1 do scale_data(iTr).wv_obs_ref = ' '
for iTr=0,Ntr-1 do scale_data(iTr).ein_A_ref = '2000APED.Z-scale..A'
;
;  Now, fit the level energy values from the Zarr, rmJarr ions
;
Lvec = fltarr(n_elements(Zarr))
Avec = fltarr(n_elements(Zarr))
for iTr=0,Ntr-1 do begin
    ll = scale_data(iTr).lower_lev
    lu = scale_data(iTr).upper_lev

    for iZ=0,n_elements(Zarr)-1 do Lvec(iZ)=$
      calc_wavelength(Zarr(iZ),rmJarr(iZ),ll,lu,/silent)*(Zarr(iZ)^2)
    gp = where(Lvec gt 0.0,count)
    if (count lt 2) then begin
        print,'Warning...one or fewer interpolation elements for wavelength.'
    endif

    c = poly_fit(Zarr(gp), Lvec(gp), 1)
;    plot,Zarr,Lvec,/ynozero
;    r = get_kbrd(1)
    scale_data(iTr).wavelen  = poly(Z, c)/(Z^2)
    scale_data(iTr).wave_obs = poly(Z, c)/(Z^2)

    for iZ=0,n_elements(Zarr)-1 do Avec(iZ)=$
      calc_einstein_A(Zarr(iZ),rmJarr(iZ),ll,lu,/silent)
    Z4 = 1.0d0*Zarr*Zarr*Zarr*Zarr
    gp = where(Avec gt 0.0,count)
    if (count lt 2) then begin
        print,'Warning...one or fewer interpolation elements for A value.'
    endif
    c = poly_fit(Zarr(gp), Avec(gp)/Z4, 1)
    scale_data(iTr).Einstein_A = poly(Z*1.d0, c)*((1.d0*Z)^4)
;    plot,Zarr(gp),Avec(gp),/ynozero
;    r = get_kbrd(1)
endfor
;
;  Write out results
;
ions_used = ' '
for iZ=0,n_elements(Zarr)-1 do $
  ions_used = ions_used+' '+roman_element(Zarr(iZ),rmJarr(iZ))
ions_used = strtrim(strcompress(ions_used),2)

comments = ['Approximated via fitting wavelength data from',ions_used,$
            'Using program zscale_wgfa.pro']

write_eline, outfile, Z, rmJ, scale_data, COMMENTS=comments, NANVALUE=nanvalue
return
end
