%
% This file provides the actual model definition and calculation of 
% the model spectrum. This routine should tie together all input and
% free variables.
%
% Parts of this code have been taken from a simpler version of the 
% apec model written by Randall Smith (SAO).
%
% Ronnie Hoogerwerf, SAO, 04/10/2002
%

uses("SL-str","SL-io");
define AM_broaden_lines();

%(lo,hi) = isis->linear_grid(12,12.4,200);
%apec(0.6,.35,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,lo,hi);


define in(elem, array) {
   
   variable indx;

   indx = where(abs(array - elem) < 0.00001);

   if (length(indx) == 0) {
      return 0;
   } else {
      return 1;
   }

}

define apec() {  
   % T1, T2, T3, T4, T5, 
   % norm1, norm2, norm3, norm4, norm5,
   % abundC, abundN, abundO, abundF, abundNe,
   % abundMg, abundAl, abundSi, abundS, abundAr,
   % abundCa, abundFe, abundNi, ContinuumFile, FitMode,
   % binlo, binhi
   
   variable nargs = 2 + AM->npar;
   variable binlo, binhi;
   variable inpbinlo, inpbinhi, index, i, T, norm, subargs;
   variable nabund = nargs - 2 - (2*AM->N_Temp_comp_max);
   variable dataset, str_dataset;
   variable fitmode, continuumfile;

   % first check if the routine is called with the correct number
   % of arguments.
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      message(AM->apec_usage_message);
      _pop_n(_NARGS);
      return NULL;
   }

   % Temporary fix to circumvent a bug in CIAO 3.0.1
   %variable savestack = NULL;
   %if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   %variable _dataset_ = get_dataset_number();
   %variable _wvl_ = get_wave_axes(_dataset_);
   %if (savestack != NULL) __push_args(savestack);

   if (_NARGS == nargs-1) {
      %
      % This model is meant to run in the "integrate ON" so this is 
      % a cheap fix to change off to on
      %
      % get parameters from stack

      inpbinlo = ();
      %
      %fitmode = ();
      %continuumfile = ();
      %
      subargs = __pop_args(nabund);
      AM->Abundances = Utils->array_from_args(__push_args(subargs));
      %
      subargs = __pop_args(AM->N_Temp_comp_max);
      norm = Utils->array_from_args(__push_args(subargs));
      %
      subargs = __pop_args(AM->N_Temp_comp_max);
      T = Utils->array_from_args(__push_args(subargs));

      % check if the wavelength arrays "binlo" and "binhi" are in
      % ascending order. If not fix it.
      if (length(inpbinlo) == 1) {
	 return [0];
      } else if (inpbinlo[1] < inpbinlo[0]) {
	 index = [length(inpbinlo)-1:0:-1];
      } else {
	 index = [0:length(inpbinlo)-1:1];
      }
      binlo = inpbinlo[index];
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      for(i = 0; i < length(binlo)-1; i++) binhi[i] = binlo[i+1];
      binhi[length(binlo)-1] = binlo[length(binlo)-1]+0.001;

   } else if (_NARGS == nargs) {
      % get parameters from stack

      inpbinhi = ();
      inpbinlo = ();
      %
      %fitmode = ();
      %continuumfile = ();

      subargs = __pop_args(nabund);
      AM->Abundances = Utils->array_from_args(__push_args(subargs));
      %
      subargs = __pop_args(AM->N_Temp_comp_max);
      norm = Utils->array_from_args(__push_args(subargs));
      %
      subargs = __pop_args(AM->N_Temp_comp_max);
      T = Utils->array_from_args(__push_args(subargs));

      % check if the wavelength arrays "binlo" and "binhi" are in
      % ascending order. If not fix it.
      if (length(inpbinlo) == 1) {
	 return [0];
      } else if (inpbinhi[0] < inpbinlo[0]) {
	 index = [length(inpbinlo)-1:0:-1];
      } else if (inpbinlo[1] < inpbinlo[0]) {
	 index = [length(inpbinlo)-1:0:-1];
      } else {
	 index = [0:length(inpbinlo)-1:1];
      }
      binlo = inpbinlo[index];
      binhi = inpbinhi[index];

   }

   %vmessage("===> %d",length(inpbinlo));
   %for (i=0;i<length(binlo);i++) {
   %   printf("%d %f  %f\n",i, binlo[i],binhi[i]);
   %}

   %
   % Temporary fix to circumvent a bug in CIAO 3.0.1
   variable _dataset_ = get_dataset_number();
   str_dataset = string(_dataset_);
   if (AM->verbose == 1) vmessage("%d %e %e %e", _dataset_, T[0], norm[0], AM->Abundances[0]);

   % Temporary until get_mdl_models() is fixed.
   if (AM->getmdlmodels != NULL) {
      variable tmp = _typeof(binlo)[length(binlo)]; 
      tmp[*] = 0.0; 
      return tmp;
   }
   % End temporary fix


   variable component = Struct_Type[AM->N_Temp_comp];

   for (i = 0; i < AM->N_Temp_comp; i++) {

      % Get a default plasma model structure from ISIS
      component[i] = isis->default_plasma_state();

      % Modify the structure to reflect the apec model requested
      component[i].temperature = T[i]/AM->K_to_keV;  % convert from keV to K
      component[i].norm = norm[i];
      component[i].metal_abund = 0.0;
      component[i].elem = AM->ElementsZ;
      component[i].elem_abund = AM->Abundances;

   }

   isis->define_model(component);

   variable flux, tmp_flux = NULL;
   
   %
   %
   variable lines;
   switch(AM->Fit)
   { case AM->Fit_Continuum_only : 
	tmp_flux = isis->model_spectrum(binlo, binhi, isis->MODEL_CONTIN);}
   { case AM->Fit_Lines_only : 
         if (AM->fix == NULL and AM->remove == NULL) {
	    tmp_flux = isis->model_spectrum(binlo, binhi, isis->MODEL_LINES);
	 } else {
	    lines = AM_get_free_lines(_dataset_);
	    if (lines != NULL) {
	       tmp_flux = isis->model_spectrum(binlo, binhi, isis->MODEL_LINES, lines);
	    } else {
	       tmp_flux = isis->model_spectrum(binlo, binhi, isis->MODEL_LINES);
	    }
	 }}
   { case AM->Fit_Continuum_and_lines : 
	if (AM->fix == NULL and AM->remove == NULL) {
	   tmp_flux = 
	       isis->model_spectrum(binlo,binhi,isis->MODEL_LINES_AND_CONTINUUM);
	} else {
	   lines = AM_get_free_lines(_dataset_);
	   if (lines != NULL) {
	      tmp_flux = isis->model_spectrum(binlo,binhi,isis->MODEL_LINES_AND_CONTINUUM,lines);
	   } else {
	      tmp_flux = isis->model_spectrum(binlo,binhi,isis->MODEL_LINES_AND_CONTINUUM);
	   }
	}}
   { case AM->Continuum_only : 
	 if (AM->continuum == NULL) {
	    error("AM_model: No continuum specified!");
	 } else {
	    tmp_flux = _typeof(binlo)[length(binlo)]; 
	    tmp_flux[*] = 0.0; 
	 }}
   { tmp_flux = isis->model_spectrum(binlo, binhi);}
   %
   % End switch(AM->Fit)
   if (AM->fix != NULL) tmp_flux += AM_get_fixed_line_flux(_dataset_,binlo,binhi);

   %
   %
    IO->write_block("flux.txt",binlo,binhi,tmp_flux);
   %
   %

   %
   % Now add the continuum if one is specified. This is slightly tricky 
   % because one fitting run might deal with spectra of different bin sizes.
   % We thus need to keep track of what spectrum we are using and link it
   % to the appropriate continuum. We store these continua to minimize 
   % run-time.
   % This facility is only available when the AM->Fit flag is equal to
   % AM->Lines_only or AM->Continuum_only.

   if (andelse
       {NULL != AM->continuum}                          % is there a cont.
       {assoc_key_exists(AM->continuum,"0") or          % for all datasets or
	assoc_key_exists(AM->continuum,str_dataset)}    % for this part. one
       {(AM->Fit == AM->Fit_Lines_only) or              % are we in the 
	(AM->Fit == AM->Continuum_only)}) {             % calc. mode

      % a continuum needs to be added to the model spectrum.
      
      variable continuum = NULL;
      variable current_nbin = length(binlo);
      variable current_data = AM_get_data(_dataset_);

      if (1 == AM_compare_bins(current_data,
			       AM->continua[_dataset_].datarange)) {
	 continuum = AM->continua[_dataset_].continuum;
      } else {
	 % The following happens only once to initialize the appropriate
	 % continuum fluxes.
	 if (assoc_key_exists(AM->continuum,"0")) {
	    isis->define_model(AM->continuum["0"]);
	 } else {
	    isis->define_model(AM->continuum[str_dataset]);
	 }
	 continuum = isis->model_spectrum(binlo,binhi,isis->MODEL_CONTIN);
	 
	 AM->continua[_dataset_].continuum = continuum;
	 AM->continua[_dataset_].datarange = current_data;
      }

      % add continuum to model
      tmp_flux += continuum;

   }

   %
   % If requested and if available broader the lines (or in other words)
   % convolve the spectum.
   %

   %tmp_flux = AM_broaden_lines(tmp_flux, _dataset_);

   % Make sure that when wavelength came in on descending order that 
   % we flip the flux so take it matches the wavelength scale used in
   % Sherpa.

   % Temporary fix to circumvent a bug in CIAO 3.0.1
   %for (i = 0; i < length(binlo); i++) {
   %   if (not in(binlo[i],_wvl_.lo)) {
   %	 tmp_flux[i] = 0.0;
   %   }
   %}

   %IO->write_block("exhydra-apec-cont-"+string(_dataset_)+".txt",binlo,binhi,tmp_flux);

   flux = tmp_flux[index];


   if (Float_Type == _typeof(inpbinlo)) {
      % message("Conversion to Float_Type");
      flux = typecast(flux, Float_Type);
   }

   return flux;

}


define check_dataset_range(str) {

   %
   % check format of the dataset_range argument of SM_source(). It should be
   % of the form '#:#,#:#,...'.  ':' and ',' may be omitted. So valid forms
   % are :   #
   %         #,#,#
   %         #:#
   %         #:#,#:#
   %         #,#:#
   %         #:#,#
   % white spaces should be omitted.
   %
   
   if (String_Type != typeof(str)) {
      message("dataset_range argument in AM_source(dataset_range,modelname) should");
      vmessage("  be a string or integer; received a %s", Utils->print_typeof(str));
      return NULL;
   }


   variable str_nws = Str->remove_whitespace(str);
   variable elems = strchop(str_nws,',',0);
   variable i, j, elem;
   
   for (i = 0; i < length(elems); i++) {
      for (j = 0; j < strlen(elems[i]); j++) {
	 if ((not (Str->is_digit(elems[i][j]))) and
	     (elems[i][j] != ':')) {
	    return 0;
	 }
      }
   }

   return 1;

}

define AM_parse_dataset_range (str) {

   variable result = Integer_Type[AM->max_datasets];

   if (0 == check_dataset_range(str)) {
      return NULL;
   } else {

      variable str_nws = Str->remove_whitespace(str);
      variable elems = strchop(str_nws,',',0);
      variable i, j, k, elem1, elem2;
      
      k = 0;
      for (i = 0; i < length(elems); i++) {
	 if (NULL == Str->position(':',elems[i])) {
	    () = sscanf(elems[i],"%d",&elem1);
	    result[k] = elem1; k++;
	 } else {
	    () = sscanf(elems[i],"%d:%d",&elem1,&elem2);
	    for (j = elem1; j <= elem2; j++) {
	       result[k] = j; k++;
	    }
	 }
      }
      return result[where(result != 0)];
   }

}


define AM_make_model() {

   % 
   % Use "sherpa_register_model" command to register the apec model with Sherpa
   % so that it will become available as a user model in sherpa and can be used
   % for fitting data.
   %

   if (is_defined("register_model") == 0) {
      error("You are using an old version of ciao. Please update to a more recent ciao version");
   }

   % Now depending on the definition in AM->standard_model_file or the user specific
   % AM->model_file create the apec model.
   %
   variable i, n = AM->N_Temp_comp_max;
   variable cmd = "() = register_model(\"apec\",[";     % Model name

   % Parameter names	
   for (i = 1; i <= n; i++) cmd = cmd + "\"T" + string(i) + "\",";
   for (i = 1; i <= n; i++) cmd = cmd + "\"norm" + string(i) + "\",";
   for (i = 0; i < AM->nelem-1; i++) {
      cmd = cmd + "\"" + AM->abunds[i] + "\",";
   }
   cmd = cmd + "\"" + AM->abunds[AM->nelem-1] + "\"],";

   % This is a 1-dimensional model
   cmd = cmd + "1,";

   % Default values
   cmd = cmd + "[";
   cmd = cmd + "AM->Tdef,";
   for (i = 2; i <= n; i++) cmd = cmd + "1.0,";
   cmd = cmd + "AM->Ndef,";
   for (i = 2; i <= n; i++) cmd = cmd + "0.0,";
   for (i = 0; i < AM->nelem-1; i++) {
      cmd = cmd + "AM->Adef,";
   }
   cmd = cmd + "AM->Adef],";

   % Minimum values
   cmd = cmd + "[";
   for (i = 1; i <= n; i++) cmd = cmd + "AM->Tmin,";
   for (i = 1; i <= n; i++) cmd = cmd + "AM->Nmin,";
   for (i = 0; i < AM->nelem-1; i++) {
      cmd = cmd + "AM->Amin,";
   }
   cmd = cmd + "AM->Amin],";

   % Maximum values
   cmd = cmd + "[";
   for (i = 1; i <= n; i++) cmd = cmd + "AM->Tmax,";
   for (i = 1; i <= n; i++) cmd = cmd + "AM->Nmax,";
   for (i = 0; i < AM->nelem-1; i++) {
      cmd = cmd + "AM->Amax,";
   }
   cmd = cmd + "AM->Amax],";

   % Thaw or Freeze
   cmd = cmd + "[";
   cmd = cmd + "AM->Ttf,";
   for (i = 2; i <= n; i++) cmd = cmd + "0,";
   cmd = cmd + "AM->Ntf,";
   for (i = 2; i <= n; i++) cmd = cmd + "0,";
   for (i = 0; i < AM->nelem-1; i++) {
      cmd = cmd + "AM->Atf,";
   }
   cmd = cmd + "AM->Atf]";

   cmd = cmd + ");";
   
   % Just as a check   
   %
   % print(cmd);
   %

   eval(cmd);

}

 
