%
%
%
%
%
%

uses("SL-str","SL-ll");

define AM_sherpa_eval(cmnd) {
   
   if (sherpa_eval(cmnd) == -1) {
      vmessage("Command \"%s\" failed.", cmnd);

   }
}

define AM_get_apec_models() {

   % 
   % Shortcut to spead up calculation. This flag prevents the 
   % apec function from evaluating and just returns 0 for the
   % model flux. Since we are only interested in the modelnames
   % and dataset numbers this is a valid shortcut.
   % Furthermore at this moment (06/19/2002) there is still a 
   % bug in the get_mdl_models() function with respect to 
   % get_dataset_number().
   %
   AM->getmdlmodels = 1;

   % set verbosity to 0
   variable verbose = AM->verbose;
   AM->verbose = 0;

   variable mdl = get_mdl_models();
   variable n = length(mdl.src), i;
   variable models = Assoc_Type[LList_Type], ll;
   variable p1, p2, modelname, dataset;


   % loop over mdl structure to find apec models and store them
   % in an associated array of linked lists.
   %
   for (i = 0; i < n; i++) {
      if (0 != string_match(mdl.model[i],"apec",1)) {
	 % Hurray: Found apec model
	 % Now get its name
	 p1 = Str->position('[',mdl.model[i]);
	 p2 = Str->position(']',mdl.model[i]);
	 modelname = substr(mdl.model[i],p1+2,p2-p1-1);
	 dataset = string(mdl.src[i]);
	 if (assoc_key_exists(models,dataset) == 1) {
	    !if (LL->member(modelname,models[dataset])) { 
	       models[dataset] = LL->append(modelname,models[dataset]);
	    }
	 } else {
	    ll = @LList_Type;
	    ll.content = modelname;
	    models[dataset] = ll;
	 }
      }

   }

   AM->getmdlmodels = NULL;
   
   % set verbosity to its original value.
   AM->verbose = verbose;

   message("\n\n\nIGNORE above messages. These are needed to determine");
   message("the apec modelnames currently in use by sherpa.\n\n");

   return models;

}

define AM_get_apec_modelnames_only() {

   variable models = AM_get_apec_models();
   variable modelnames = NULL, k, vs, v;

   foreach (models) using ("keys","values") {
      (k, vs) = ();

      foreach (vs) {
	 v = ();
	 if (LL->member(v.content,modelnames) == 0) {
	     modelnames = LL->append(v.content,modelnames);
	 }
      }
   }

   modelnames = LL->ll_to_array(modelnames);

   return modelnames;      

}

define AM_compare_bins(arr1, arr2) {


   %
   % if both of the arrays are NULL return 1
   %
   if (arr2 == NULL and arr1 == NULL) return 1;
   %
   % if one of the arrays is NULL return 0
   %
   if (NULL == arr2 or NULL == arr1) return 0;
   %
   % if the lengths are not the same return 0
   %
   if (length(arr1) != length(arr2)) return 0;

   %
   % Compare element by element using where.
   %
   if (length(where(arr1 != arr2)) == 0) {
      return 1;
   } else {
      return 0;
   }

}

define AM_get_version() {

   return "Apec Models version " + AM->AM_version;

}

define AM_print_version() {

   vmessage(AM_get_version());

}

define AM_strZ(Z) {

   switch(Z)
   { case  1 : return "H"; }
   { case  2 : return "He"; }
   { case  3 : return "Li"; }
   { case  4 : return "Be"; }
   { case  5 : return "B"; }
   { case  6 : return "C"; }
   { case  7 : return "N"; }
   { case  8 : return "O"; }
   { case  9 : return "F"; }
   { case 10 : return "Ne"; }
   { case 11 : return "Na"; }
   { case 12 : return "Mg"; }
   { case 13 : return "Al"; }
   { case 14 : return "Si"; }
   { case 15 : return "P"; }
   { case 16 : return "S"; }
   { case 17 : return "Cl"; }
   { case 18 : return "Ar"; }
   { case 19 : return "K"; }
   { case 20 : return "Ca"; }
   { case 21 : return "Sc"; }
   { case 22 : return "Ti"; }
   { case 23 : return "V"; }
   { case 24 : return "Cr"; }
   { case 25 : return "Mn"; }
   { case 26 : return "Fe"; }
   { case 27 : return "Co"; }
   { case 28 : return "Ni"; }
   { case 29 : return "Cu"; }
   { case 30 : return "Zn"; }
   { return "??"; }
}

define AM_strIon(ion) {

   switch(ion)
   { case NULL : return "All"; }
   { case  1 : return "I"; }
   { case  2 : return "II"; }
   { case  3 : return "III"; }
   { case  4 : return "IV"; }
   { case  5 : return "V"; }
   { case  6 : return "VI"; }
   { case  7 : return "VII"; }
   { case  8 : return "VIII"; }
   { case  9 : return "IX"; }
   { case 10 : return "X"; }
   { case 11 : return "XII"; }
   { case 12 : return "XII"; }
   { case 13 : return "XIII"; }
   { case 14 : return "XIV"; }
   { case 15 : return "XV"; }
   { case 16 : return "XVI"; }
   { case 17 : return "XVII"; }
   { case 18 : return "XVIII"; }
   { case 19 : return "XIX"; }
   { case 20 : return "XX"; }
   { case 21 : return "XXI"; }
   { case 22 : return "XXII"; }
   { case 23 : return "XXIII"; }
   { case 24 : return "XXIV"; }
   { case 25 : return "XXV"; }
   { case 26 : return "XXVI"; }
   { case 27 : return "XXVII"; }
   { case 28 : return "XXVIII"; }
   { case 29 : return "XXIX"; }
   { case 30 : return "XXX"; }
   { return "??"; }
}
