%
%
%
%
%
%
%
uses("SL-constants");
use_namespace("Cnst");


variable c = 299792458.0;             % Speed of light [m/s]
variable eV = 1.60218e-19;            % Electon Volt [Joule]
variable k = 1.3806503e-23;           % Boltzmann constant [J K-1]
variable k_eV = 8.617342e-5;          % Boltzmann constant [eV K-1]
variable k_Hz = 2.0836644e10;         % Boltzmann constant / h [Hz K-1]
variable h = 6.62606876e-34;          % Plancks constant [J s]
variable h_eVs = 4.13566727e-15;      % Plancks constant [eV s]
variable hbar = 1.054571596e-34;      % Hbar [J s]
variable hbar_eVs = 6.58211889e-16;   % Hbar [eV s]
variable G = 6.673e-11;               % Gravitational constant [m3 kg-1 s-2]
variable Gbar = 6.707e-39;            % Gbar (G/(hbar c)) [(GeV/c2)-2]
variable eps_o = 8.854187817e-12;     % Electric constant [F m-1]
variable mu_o = 12.566370614e-7;      % Magnetic constant [N A-2]
variable Z_o = 376.730313461;         % impedance of vacuum [Ohm]



variable Distance = Assoc_Type[];
Distance["m"]         = 1.0;
Distance["km"]        = 1.0e+3;
Distance["hm"]        = 1.0e+2;
Distance["dm"]        = 1.0e-1;
Distance["cm"]        = 1.0e-2;
Distance["mm"]        = 1.0e-3;
Distance["micron"]    = 1.0e-6;
Distance["nm"]        = 1.0e-9;
Distance["angstrom"]  = 1.0e-10;
Distance["lightyear"] = 9.46073047e15;
Distance["parsec"] = 3.26 * Distance["lightyear"];



define Angstrom_to_eV (A) {

   variable ev = (h * c) / (A * Distance["angstrom"]);  % in Joule
   
   ev = ev / Cnst->eV;

   return ev;

}

define eV_to_Angstrom (ev) {

   variable A = (h * c) / (ev * Cnst->eV);  % in meters
   
   A = A / Cnst->Distance["angstrom"];

   return A;

}
