%
%
%
%
%
%
%
%
% Ronnie Hoogerwerf, SAO, 05/07/2002
%
uses("SL-ll","SL-utilities","SL-str");
use_namespace("LL");



%----

static define ll_to_array(ll) {
   
   variable type, i, n, array;

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      error("LL->ll_to_array(ll): ll is not of type LList_Type");
   }

   % handle empty list
   if (ll == NULL) return ll;

   n = LL->length(ll);
   type = typeof(ll.content);

   % check if all elements of ll are of the same type
   for (i = 0; i < n; i++) {
      !if (typeof(LL->nth(i,ll)) == type) {
	 error("LL->ll_to_array(ll): not all elements of ll are of the same type");
      }
   }       

   array = @Array_Type(type,n);
   for (i = 0; i < n; i++) array[i] = LL->nth(i,ll);

   return array;

}

%----

static define member(obj,ll) {

   variable tmp = ll;

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      error("LL->member(obj,ll): ll is not of type LList_Type");
   }

   % handle empty list
   if (ll == NULL) return 0;

   while (tmp.next != NULL) {
      if (Utils->equal(tmp.content, obj)) return 1;
      tmp = tmp.next;
   }

   if (Utils->equal(tmp.content, obj)) return 1;

   return 0;

}

%----

static define nth(nth,ll) {

   variable tmp, i = 0;

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      error("LL->replace_nth(nth,obj,ll): ll is not of type LList_Type");
   }

   if (ll == NULL) error("LL->replace_nth(nth,obj,ll): ll is an empty list");

   if (nth >= LL->length(ll)) {
      error("LL->replace_nth(nth,obj,ll): nth is larger than lenght of ll");
   }

   tmp = ll;
   while (tmp.next != NULL and i < nth) {
      tmp = tmp.next;
      i++;
   }

   return tmp.content;

}

%----

static define remove(nth,ll) {

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      return NULL;
   }

   % handle empty list
   if (ll == NULL) return ll;

   variable n = length(ll), tmp = ll, i, new;

   if (n == 1 and nth == 0) {
      ll = NULL;
   } else if (nth >= n-1) {
      % remove last element
      tmp = ll;
      for(i = 0 ; i < n-2; i++) tmp = tmp.next;
      tmp.next = NULL;
   } else if (nth == 0) {
      % remove first element
      tmp = ll;
      ll = tmp.next;
   } else {
      for(i = 0 ; i < nth - 1; i++) tmp = tmp.next;
      new = tmp;
      new = new.next; new = new.next;
      tmp.next = new;
   }
       
   return ll;

}

%----

static define insert(nth,obj,ll) {


   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      return NULL;
   }

   variable n = LL->length(ll), tmp, i, new_begin, new_end;

   if (nth > n-1) {
      % append obj to the right
      ll = right_append(obj,ll);
   } else if (nth == 0) {
      % append obj to the left
      ll = left_append(obj,ll);
   } else {
      tmp = ll;
      for (i = 0; i < nth-1; i++) tmp = tmp.next;
      if (typeof(obj) == LList_Type) {
	 new_begin = obj;
	 new_end = obj;
	 while(new_end.next != NULL) new_end = new_end.next;
	 new_end.next = tmp.next;
	 tmp.next = new_begin;
      } else {
	 new_begin = @LList_Type;
	 new_begin.content = obj;
	 new_begin.next = tmp.next;
	 tmp.next = new_begin;
      }
   }

   return ll;

}

%----

static define right_append(obj,ll) {

   ll = append(obj,ll);

   return ll;

}

%----

define append(obj,ll) {

   variable tmp = ll, new = @LList_Type;

   if (ll == NULL) {
      if (typeof(obj) == LList_Type) {
	 ll = obj;
      } else {
	 ll = @LList_Type;
	 ll.content = obj;
	 ll.next = NULL;
      }
      return ll;
   }

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      error("LL->append(obj,ll): ll is not of type LList_Type");
   }
   
   while (tmp.next != NULL) tmp = tmp.next;

   if (typeof(obj) == LList_Type) {
      tmp.next = obj;
   } else {
      new.content = obj;
      new.next = NULL;
      tmp.next = new;
   }
       
   return ll;

}

%----

static define left_append(obj,ll) {

   variable tmp, new, oldll = ll;

   if (LList_Type != typeof(oldll) and Null_Type != typeof(ll)) {
      error("LL->append(obj,ll): ll is not of type LList_Type");
   }
   
   % handle empty list
   if (typeof(ll) == Null_Type) {
      if (typeof(obj) == LList_Type) {
	 ll = obj;
      } else {
	 new = @LList_Type;
	 new.content = obj;
	 new.next = NULL;
	 ll = new;
      }
   } else if (typeof(obj) == LList_Type) {
      ll = obj;
      tmp = obj;
      while (tmp.next != NULL) tmp = tmp.next;
      tmp.next = oldll;
   } else {
      new = @LList_Type;
      new.content = obj;
      new.next = oldll;
      ll = new;
   }
       
   return ll;

}

%----

static define ll_from_args() {
   
   variable ll_begin, ll_end, next, content, i;
   
   ll_begin = NULL;

   for (i = 1; i <= _NARGS; i++) {
      content = ();
      
      next = @LList_Type;
      next.content = content;
      next.next = NULL;

      if (ll_begin == NULL) {
	 % first element; start the list
	 ll_begin = next;
      } else {
	 % additional elements need to be appended
	 ll_end.next = next;
      }

      ll_end = next;

   }

   return LL->reverse(ll_begin);
}

%----

static define replace_nth(nth,obj,ll) {

   variable tmp, i = 0;

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      error("LL->replace_nth(nth,obj,ll): ll is not of type LList_Type");
   }

   % handle empty list
   if (ll == NULL) {
      error("LL->replace_nth(nth,obj,ll): ll is a NULL list");
   }

   if (nth >= LL->length(ll)) {
      error("LL->replace_nth(nth,obj,ll): nth is larger than lenght of ll");
   }

   tmp = ll;
   while (tmp.next != NULL and i < nth) {
      tmp = tmp.next;
      i++;
   }

   tmp.content = obj;

   return ll;

}

%----

static define create_empty(n) {

   variable ll_begin = NULL, ll_end, next, i;

   if (n < 1) {
      error("create_empty(n): n is less than 1");
   }

   for (i = 1; i <= n; i++) {
      next = @LList_Type;
      next.content = 0;
      next.next = NULL;
      if (ll_begin == NULL) {
	 ll_begin = next;
      } else {
	 ll_end.next = next;
      }

      ll_end = next;
   }

   return ll_begin;

}

%----

static define reverse(ll) {

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      return NULL;
   }

   % handle empty list
   if (ll == NULL) return ll;

   variable n = LL->length(ll);
   variable tmp, new = create_empty(n), i;

   tmp = ll;

   for (i = 1; i <= n; i++) {
      new = replace_nth(n-i,tmp.content,new);
      tmp = tmp.next;
   }

   return new;

}

%----

static define print(ll) {

   variable format, tmp;

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      return;
   }

   % handle empty list
   if (ll == NULL) {
      ()= printf("Null list");
      return;
   }

   tmp = ll;
   () = printf("(");
   while (tmp.next != NULL) {
      format = get_format(tmp.content) + ",";
      () = printf(format,tmp.content);
      tmp = tmp.next;
   }
   format = get_format(tmp.content);
   () = printf(format,tmp.content);

   () = printf(")\n");

}

%----

static define length(ll) {

   variable tmp, n = 1;

   if (LList_Type != typeof(ll) and Null_Type != typeof(ll)) {
      return NULL;
   }

   % handle empty list
   if (ll == NULL) return 0;

   tmp = ll;
   while (tmp.next != NULL) {
      tmp = tmp.next;
      n++;
   }

   return n;

}

%----

private define get_format (obj) {

   switch (typeof(obj)) 
   { case Char_Type     : return "%c"; }
   { case UChar_Type    : return "%c"; }
   { case Short_Type    : return "%d"; }
   { case UShort_Type   : return "%d"; }
   { case Integer_Type  : return "%d"; }
   { case UInteger_Type : return "%d"; }
   { case Long_Type     : return "%d"; }
   { case ULong_Type    : return "%d"; }
   { case Float_Type    : return "%e"; }
   { case Double_Type   : return "%e"; }
   { case Complex_Type  : error("No format available for Complex_Type"); }
   { case String_Type   : return "%s"; }
   { case BString_Type  : error("No format available for BString_Type"); }
   { case Struct_Type   : error("No format available for Struct_Type"); }
   { case Ref_Type      : error("No format available for Ref_Type"); }
   { case Null_Type     : error("No format available for Null_Type"); }
   { case Array_Type    : error("No format available for Array_Type"); }
   { case DataType_Type : error("No format available for DataType_Type"); }

}

