%
% General purpose SLang routines
%
%
%
%
uses("SL-str","SL-ll","SL-utilities");
use_namespace("Str");


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Stuff to with character definitions.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

define is_whitespace(char) {

   if (orelse {char == ' '} {char == '\t'} {char == '\n'}) {
      return 1;
   } else {
      return 0;
   }

}

define is_alphabetic(char) {

   %
   % is char a member of:
   %
   % a b c d e f g h i j k l m n o p q r s t u v w x y z
   % A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
   %

   if (((char >= 'a') and (char <= 'z')) or
       ((char >= 'A') and (char <= 'Z'))) {
      return 1;
   } else {
      return 0;
   }

}

define is_numeric(char) {

   % is char a member of 
   %
   % 0 1 2 3 4 5 6 7 8 9
   %

   if ((char >= '0') and (char <= '9')) {
      return 1;
   } else {
      return 0;
   }

}

define is_digit(char) {

   % is char a member of 
   %
   % 0 1 2 3 4 5 6 7 8 9
   %
   % This function should be more complicated FIXME later.

   if ((char >= '0') and (char <= '9')) {
      return 1;
   } else {
      return 0;
   }

}
%
% Use SLang intrinsic function isdigit(). Faster excecution.
%
%define is_digit(char) {
%
%   if (0 != isdigit(char)) {
%      return 1;
%   } else {
%      return 0;
%   }
%}

define is_graphic(char) {

   % is char a graphic character?
   %
   % all numeric + alpahbetic + the following:
   %
   %  32 space   33 !      34 "      (")
   %  35 #       36 $      37 %
   %  38 &       39 ' (')  40 (  
   %  41 )       42 *      43 +
   %  44 ,       45 -      46 .
   %  47 /       58 :      59 ;
   %  60 <       61 =      62 >
   %  63 ?       64 @      91 [
   %  92 \       93 ]      94 ^
   %  95 _       96 ` (`) 123 {
   % 124 |      125 }     126 ~

   if (is_digit(char) or
       %
       is_alphabetic(char) or
       %
       (((char >= 32) and (char <= 47)) or
	((char >= 58) and (char <= 64)) or
	((char >= 91) and (char <= 96)) or
	((char >= 123) and (char <= 126)))) {
      return 1;
   } else {
      return 0;
   }
}

define is_standardchar(char) {

   %
   % Is char a standard character (defined as all graphic chars + newline)
   %

   if (is_graphic(char) or (char == 10)) {
      return 1;
   } else {
      return 0;
   }
}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Stuff to do with string manipulation
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


define empty_string(n) {

   variable i, one = " ", str = "";
   
   if (n <= 0) {
      return "";
   } else {
      for (i = 1; i <= n; i++) str = str + one;
      return str;
   }
}

define split_and_trim(line,split) {

   variable i, words = strchop(line,split,0);

   for (i = 0; i < length(words); i++) words[i] = strtrim(words[i]);

   return words;

}

define remove_whitespace(str) {

   if (String_Type != typeof(str)) {
      message("Utils->remove_whitespace(str) takes a string as the argument");
      vmessage("  receives a %s", Utils->print_typeof(str));
      return NULL;	  
   }

   %define array to hold substring and initialize all elements to be empty strings
   variable n = strlen(str);
   variable str_array = String_Type[n];
   variable i, ilast, j;
   str_array[*] = "";
   
   j = 0;
   ilast = 0;
   for (i = 0; i < n; i++) {
      if (is_whitespace(str[i])) {
	 if (ilast != i) str_array[j] = str[[ilast:i-1]];
	 ilast = i + 1;
	 j++;    
      }
   }
  if (ilast != i)  str_array[j] = str[[ilast:i-1]];

  return strjoin(str_array,"");

}

define remove_excess_whitespace(str) {

   if (String_Type != typeof(str)) {
      message("Utils->remove_whitespace(str) takes a string as the argument");
      vmessage("  receives a %s", Utils->print_typeof(str));
      return NULL;	  
   }

   %define array to hold substring and initialize all elements to be empty strings
   variable n = strlen(str);
   variable str_array = String_Type[n];
   variable i, ilast, j;
   str_array[*] = "";
   
   j = 0;
   ilast = 0;
   for (i = 0; i < n; i++) {
      if (is_whitespace(str[i])) {
	 if (ilast != i) str_array[j] = str[[ilast:i]];
	 ilast = i + 1;
	 j++;    
      }
   }
  if (ilast != i)  str_array[j] = str[[ilast:i-1]];

  return strtrim(strjoin(str_array,""));

}


define position () {

   % 
   % Find the first occurence of 'elem' in the object 'str' with the
   % boundary conditions that we only look between 'start' and 'end'.
   % Mimics lisp function.
   %


   variable usage = "Usage: Str->position(elem,string [,start [, end]])";
   variable i;
   variable elem, str, start = 0, end = NULL;

   if (0 == Utils->check_nargs(_NARGS,2,3,4)) {
      message(usage);
   }

   switch (_NARGS) 
   { case 2: (elem, str) = (); }
   { case 3: (elem, str, start) = (); }
   { case 4: (elem, str, start, end) = (); }

   if (NULL == end) {
      if (String_Type == typeof(str)) {
         end = strlen(str);
      } else if (Array_Type == typeof(str)) {
         end = length(str);
      }
   }

   for (i = start; i < end; i++) {
      if (elem == str[i]) return i;
   }
   
   return NULL;

}

define rposition () {

   % 
   % Find the first occurence of 'elem' in the object 'str' with the
   % boundary conditions that we only look between 'start' and 'end'.
   % search from the back to the front.
   % Mimics lisp function.
   %


   variable usage = "Usage: Str->rposition(elem,string [,start [, end]])";
   variable i;
   variable elem, str, start = 0, end = NULL;

   if (0 == Utils->check_nargs(_NARGS,2,3,4)) {
      message(usage);
   }

   switch (_NARGS) 
   { case 2: (elem, str) = (); }
   { case 3: (elem, str, start) = (); }
   { case 4: (elem, str, start, end) = (); }

   if (NULL == end) {
      if (String_Type == typeof(str)) {
         end = strlen(str);
      } else if (Array_Type == typeof(str)) {
         end = length(str);
      }
   }

   for (i = end-1; i >= start; i--) {
      if (elem == str[i]) return i;
   }
   
   return NULL;

}


define strchop_mult() {
   
   variable usage = "Usage: strchop_mult(str,split,...)";
   variable i, j, k, n, m, str, splits;
   variable elements = NULL, tmp1, tmp2;

   if (_NARGS < 2) {
      message(usage);
   }
 
   % read string and split characters
   n = _NARGS - 1;
   splits = Integer_Type[n];
   for(i = 0; i < n; i++) splits[i] = ();
   splits = reverse(splits);
   str = ();

   % split string and collect results in a linked list.
   for (i = 0; i < n; i++) {
      m = LL->length(elements);
      !if (typeof(elements) == LList_Type) {
	 tmp2 = strchop(str,splits[i],0);
	 for (k = 0; k < length(tmp2); k++)
	     !if (strlen(tmp2[k]) == 0) {
		tmp2[k] = remove_excess_whitespace(tmp2[k]);
		elements = LL->append(tmp2[k],elements);
	     }
      } else {
	 tmp1 = elements; elements = NULL;
	 for (j = 0; j < m; j++) {
	    tmp2 = strchop(LL->nth(j,tmp1),splits[i],0);
	    for (k = 0; k < length(tmp2); k++) {
	       !if (strlen(tmp2[k]) == 0) {
		  tmp2[k] = remove_excess_whitespace(tmp2[k]);
		  elements = LL->append(tmp2[k],elements);
	       }
	    }
	 }
      }
   }

   m = LL->length(elements);
   variable results = String_Type[m];
   for (i = 0; i < m; i++) results[i] = LL->nth(i,elements);

   return results;

}

