/** \file TableFilePtr.cxx
    \brief Implementation of TableFilePtr class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <sstream>
#include <stdexcept>

#include "TableFilePtr.h"
#include "profit/Table.h"

namespace profit {

  TableFilePtr::TableFilePtr(const std::string & file_name, const std::string & table_name):
    m_col_lookup(), m_file_name(file_name), m_table_name(table_name), m_num_rows(0) {}

  TableFilePtr::~TableFilePtr() {}

  const std::string & TableFilePtr::getFileName() const { return m_file_name; }

  TableFilePtr::size_type TableFilePtr::getNumRecords() const { return m_num_rows; }

  std::string TableFilePtr::getKeyword(const std::string & key_name) const { return ""; }

  Record TableFilePtr::getRecord(TableFilePtr::size_type record_index) const {
    if (record_index >= m_num_rows) {
      std::ostringstream os;
      os << "Cannot get record number " << record_index << " in table " << m_table_name << " in file " <<
        m_file_name;
      throw std::runtime_error(os.str());
    }
    return Record(this, record_index);
  }
     
}
